import sys
from _typeshed import StrOrBytesPath, SupportsRead
from _typeshed.importlib import LoaderProtocol, MetaPathFinderProtocol, PathEntryFinderProtocol
from collections.abc import Callable, Iterable, Iterator
from typing import IO, Any, NamedTuple, TypeVar
from typing_extensions import deprecated

__all__ = [
    "get_importer",
    "iter_importers",
    "walk_packages",
    "iter_modules",
    "get_data",
    "read_code",
    "extend_path",
    "ModuleInfo",
]
if sys.version_info < (3, 14):
    __all__ += ["get_loader", "find_loader"]
if sys.version_info < (3, 12):
    __all__ += ["ImpImporter", "ImpLoader"]

_PathT = TypeVar("_PathT", bound=Iterable[str])

class ModuleInfo(NamedTuple):
    module_finder: MetaPathFinderProtocol | PathEntryFinderProtocol
    name: str
    ispkg: bool

def extend_path(path: _PathT, name: str) -> _PathT: ...

if sys.version_info < (3, 12):
    @deprecated("Deprecated since Python 3.3; removed in Python 3.12. Use the `importlib` module instead.")
    class ImpImporter:
        def __init__(self, path: StrOrBytesPath | None = None) -> None: ...

    @deprecated("Deprecated since Python 3.3; removed in Python 3.12. Use the `importlib` module instead.")
    class ImpLoader:
        def __init__(self, fullname: str, file: IO[str], filename: StrOrBytesPath, etc: tuple[str, str, int]) -> None: ...

if sys.version_info < (3, 14):
    if sys.version_info >= (3, 12):
        @deprecated("Deprecated since Python 3.12; removed in Python 3.14. Use `importlib.util.find_spec()` instead.")
        def find_loader(fullname: str) -> LoaderProtocol | None: ...
        @deprecated("Deprecated since Python 3.12; removed in Python 3.14. Use `importlib.util.find_spec()` instead.")
        def get_loader(module_or_name: str) -> LoaderProtocol | None: ...
    else:
        def find_loader(fullname: str) -> LoaderProtocol | None: ...
        def get_loader(module_or_name: str) -> LoaderProtocol | None: ...

def get_importer(path_item: StrOrBytesPath) -> PathEntryFinderProtocol | None: ...
def iter_importers(fullname: str = "") -> Iterator[MetaPathFinderProtocol | PathEntryFinderProtocol]: ...
def iter_modules(path: Iterable[StrOrBytesPath] | None = None, prefix: str = "") -> Iterator[ModuleInfo]: ...
def read_code(stream: SupportsRead[bytes]) -> Any: ...  # undocumented
def walk_packages(
    path: Iterable[StrOrBytesPath] | None = None, prefix: str = "", onerror: Callable[[str], object] | None = None
) -> Iterator[ModuleInfo]: ...
def get_data(package: str, resource: str) -> bytes | None: ...
def resolve_name(name: str) -> Any: ...



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["load_centrality", "edge_load_centrality"]

@_dispatchable
def newman_betweenness_centrality(
    G: Graph[_Node], v=None, cutoff: bool | None = None, normalized: bool | None = True, weight: str | None = None
) -> float | dict[Incomplete, float]: ...

load_centrality = newman_betweenness_centrality

@_dispatchable
def edge_load_centrality(G: Graph[_Node], cutoff: bool | None = False): ...



import codecs
from _typeshed import ReadableBuffer

HEXDIGITS: str

def hex_encode(data: str, errors: str = "strict") -> tuple[bytes, int]: ...
def hex_decode(data: bytes, errors: str = "strict") -> tuple[str, int]: ...

class Codec(codecs.Codec):
    def encode(self, data: str, errors: str = "strict") -> tuple[bytes, int]: ...
    def decode(self, data: bytes, errors: str = "strict") -> tuple[str, int]: ...

class IncrementalEncoder(codecs.IncrementalEncoder):
    state: int
    def encode(self, data: str, final: bool = False) -> bytes: ...

class IncrementalDecoder(codecs.IncrementalDecoder):
    def decode(self, data: ReadableBuffer, final: bool = False) -> str: ...

class StreamWriter(Codec, codecs.StreamWriter): ...
class StreamReader(Codec, codecs.StreamReader): ...

def getregentry() -> codecs.CodecInfo: ...



from cmd import Cmd
from collections.abc import Callable
from typing import Any, ClassVar, TextIO

import click

class ClickCmd(Cmd):
    nocommand: ClassVar[str]
    def __init__(
        self,
        ctx: click.Context | None = None,
        on_finished: Callable[[click.Context], None] | None = None,
        hist_file: str | None = None,
        completekey: str = "tab",
        stdin: TextIO | None = None,
        stdout: TextIO | None = None,
    ) -> None: ...
    def preloop(self) -> None: ...
    def postloop(self) -> None: ...
    def cmdloop(self, intro: str | None = None) -> None: ...
    def get_prompt(self) -> str | None: ...
    def emptyline(self) -> bool: ...
    def default(self, line: str) -> None: ...
    def get_names(self) -> list[str]: ...
    def do_help(self, arg: str) -> None: ...
    def do_quit(self, arg: str) -> bool: ...
    def do_exit(self, arg: str) -> bool: ...
    def print_topics(self, header: Any, cmds: list[str] | None, cmdlen: int, maxcol: int) -> None: ...



from markdown.extensions import Extension
from markdown.inlinepatterns import UnderscoreProcessor

EMPHASIS_RE: str
STRONG_RE: str
STRONG_EM_RE: str

class LegacyUnderscoreProcessor(UnderscoreProcessor): ...
class LegacyEmExtension(Extension): ...

def makeExtension(**kwargs) -> LegacyEmExtension: ...



import asyncio
from re import Pattern
from typing import Any, ClassVar, overload
from typing_extensions import TypeAlias, deprecated

class ChannelLayerManager:
    backends: dict[str, BaseChannelLayer]

    def __init__(self) -> None: ...
    @property
    def configs(self) -> dict[str, Any]: ...
    def make_backend(self, name: str) -> BaseChannelLayer: ...
    def make_test_backend(self, name: str) -> Any: ...
    def __getitem__(self, key: str) -> BaseChannelLayer: ...
    def __contains__(self, key: str) -> bool: ...
    def set(self, key: str, layer: BaseChannelLayer) -> BaseChannelLayer | None: ...

_ChannelCapacityPattern: TypeAlias = Pattern[str] | str
_ChannelCapacityDict: TypeAlias = dict[_ChannelCapacityPattern, int]
_CompiledChannelCapacity: TypeAlias = list[tuple[Pattern[str], int]]

class BaseChannelLayer:
    MAX_NAME_LENGTH: ClassVar[int] = 100
    expiry: int
    capacity: int
    channel_capacity: _ChannelCapacityDict
    channel_name_regex: Pattern[str]
    group_name_regex: Pattern[str]
    invalid_name_error: str

    def __init__(self, expiry: int = 60, capacity: int = 100, channel_capacity: _ChannelCapacityDict | None = None) -> None: ...
    def compile_capacities(self, channel_capacity: _ChannelCapacityDict) -> _CompiledChannelCapacity: ...
    def get_capacity(self, channel: str) -> int: ...
    @overload
    def match_type_and_length(self, name: str) -> bool: ...
    @overload
    def match_type_and_length(self, name: object) -> bool: ...
    @overload
    def require_valid_channel_name(self, name: str, receive: bool = False) -> bool: ...
    @overload
    def require_valid_channel_name(self, name: object, receive: bool = False) -> bool: ...
    @overload
    def require_valid_group_name(self, name: str) -> bool: ...
    @overload
    def require_valid_group_name(self, name: object) -> bool: ...
    @overload
    def valid_channel_names(self, names: list[str], receive: bool = False) -> bool: ...
    @overload
    def valid_channel_names(self, names: list[Any], receive: bool = False) -> bool: ...
    def non_local_name(self, name: str) -> str: ...
    async def send(self, channel: str, message: dict[str, Any]) -> None: ...
    async def receive(self, channel: str) -> dict[str, Any]: ...
    async def new_channel(self) -> str: ...
    async def flush(self) -> None: ...
    async def group_add(self, group: str, channel: str) -> None: ...
    async def group_discard(self, group: str, channel: str) -> None: ...
    async def group_send(self, group: str, message: dict[str, Any]) -> None: ...
    @deprecated("Use require_valid_channel_name instead.")
    def valid_channel_name(self, channel_name: str, receive: bool = False) -> bool: ...
    @deprecated("Use require_valid_group_name instead.")
    def valid_group_name(self, group_name: str) -> bool: ...

_InMemoryQueueData: TypeAlias = tuple[float, dict[str, Any]]

class InMemoryChannelLayer(BaseChannelLayer):
    channels: dict[str, asyncio.Queue[_InMemoryQueueData]]
    groups: dict[str, dict[str, float]]
    group_expiry: int

    def __init__(
        self,
        expiry: int = 60,
        group_expiry: int = 86400,
        capacity: int = 100,
        channel_capacity: _ChannelCapacityDict | None = None,
    ) -> None: ...

    extensions: list[str]

    async def send(self, channel: str, message: dict[str, Any]) -> None: ...
    async def receive(self, channel: str) -> dict[str, Any]: ...
    async def new_channel(self, prefix: str = "specific.") -> str: ...
    async def flush(self) -> None: ...
    async def close(self) -> None: ...
    async def group_add(self, group: str, channel: str) -> None: ...
    async def group_discard(self, group: str, channel: str) -> None: ...
    async def group_send(self, group: str, message: dict[str, Any]) -> None: ...

def get_channel_layer(alias: str = "default") -> BaseChannelLayer | None: ...

channel_layers: ChannelLayerManager



from typing import Final

from .api import APIClient as APIClient
from .client import DockerClient as DockerClient, from_env as from_env
from .context import Context as Context, ContextAPI as ContextAPI
from .tls import TLSConfig as TLSConfig
from .version import __version__ as __version__

__title__: Final[str]



from .base import DictType

class Healthcheck(DictType):
    def __init__(self, **kwargs) -> None: ...
    @property
    def test(self): ...
    @test.setter
    def test(self, value) -> None: ...
    @property
    def interval(self): ...
    @interval.setter
    def interval(self, value) -> None: ...
    @property
    def timeout(self): ...
    @timeout.setter
    def timeout(self, value) -> None: ...
    @property
    def retries(self): ...
    @retries.setter
    def retries(self, value) -> None: ...
    @property
    def start_period(self): ...
    @start_period.setter
    def start_period(self, value) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import typing

import google.protobuf.descriptor
import google.protobuf.message
import tensorflow.tsl.protobuf.error_codes_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class StatusProto(google.protobuf.message.Message):
    """Wire-format for Status.
    Next tag: 3
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CODE_FIELD_NUMBER: builtins.int
    MESSAGE_FIELD_NUMBER: builtins.int
    code: tensorflow.tsl.protobuf.error_codes_pb2.Code.ValueType
    """Status code as defined in tensorflow/tsl/protobuf/error_codes.proto."""
    message: builtins.str
    """Detail error message."""
    def __init__(
        self, *, code: tensorflow.tsl.protobuf.error_codes_pb2.Code.ValueType | None = ..., message: builtins.str | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["code", b"code", "message", b"message"]) -> None: ...

global___StatusProto = StatusProto



from typing import Literal, overload

from ._typing import SupportsRead, SupportsWrite
from .geometry.base import BaseGeometry
from .lib import Geometry

def loads(data: str | bytes, hex: bool = False) -> BaseGeometry: ...
def load(fp: SupportsRead[str] | SupportsRead[bytes], hex: bool = False) -> BaseGeometry: ...
@overload
def dumps(ob: Geometry, hex: Literal[False] = False, srid: int | None = None, **kw) -> bytes: ...
@overload
def dumps(ob: Geometry, hex: Literal[True], srid: int | None = None, **kw) -> str: ...
@overload
def dump(ob: Geometry, fp: SupportsWrite[bytes], hex: Literal[False] = False, *, srid: int | None = None, **kw) -> None: ...
@overload
def dump(ob: Geometry, fp: SupportsWrite[str], hex: Literal[True], *, srid: int | None = None, **kw) -> None: ...



from _typeshed import Incomplete
from typing import Final

from docutils.parsers.rst import Directive

__docformat__: Final = "reStructuredText"

class Image(Directive):
    align_h_values: Incomplete
    align_v_values: Incomplete
    align_values: Incomplete
    loading_values: Incomplete
    def align(argument): ...
    def loading(argument): ...

class Figure(Image):
    def align(argument): ...
    def figwidth_value(argument): ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
A checkpoint file is an sstable. The value for each record is a serialized
SavedTensorSlices message (defined below).

Each checkpoint file has a record with the empty key (""), which corresponds
to a SavedTensorSlices message that contains a "meta", that serves as a
table of contents on all the tensor slices saved in this file. Since the key
is "", it's always the first record in each file.

Each of the rest of the records in a checkpoint stores the raw data of a
particular tensor slice, in SavedSlice format. The corresponding key is an
ordered code that encodes the name of the tensor and the slice
information. The name is also stored in the SaveSlice message for ease of
debugging and manual examination.
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import tensorflow.core.framework.tensor_pb2
import tensorflow.core.framework.tensor_shape_pb2
import tensorflow.core.framework.tensor_slice_pb2
import tensorflow.core.framework.types_pb2
import tensorflow.core.framework.versions_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class SavedSliceMeta(google.protobuf.message.Message):
    """Metadata describing the set of slices of the same tensor saved in a
    checkpoint file.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    SHAPE_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    SLICE_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Name of the tensor."""
    type: tensorflow.core.framework.types_pb2.DataType.ValueType
    """Type of the tensor"""
    @property
    def shape(self) -> tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto:
        """Shape of the tensor"""

    @property
    def slice(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        tensorflow.core.framework.tensor_slice_pb2.TensorSliceProto
    ]:
        """Explicit list of slices saved in the checkpoint file."""

    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        shape: tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto | None = ...,
        type: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
        slice: collections.abc.Iterable[tensorflow.core.framework.tensor_slice_pb2.TensorSliceProto] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["shape", b"shape"]) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing.Literal["name", b"name", "shape", b"shape", "slice", b"slice", "type", b"type"]
    ) -> None: ...

global___SavedSliceMeta = SavedSliceMeta

@typing.final
class SavedTensorSliceMeta(google.protobuf.message.Message):
    """Metadata describing the set of tensor slices saved in a checkpoint file.
    It is always stored at the beginning of each checkpoint file.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TENSOR_FIELD_NUMBER: builtins.int
    VERSIONS_FIELD_NUMBER: builtins.int
    @property
    def tensor(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___SavedSliceMeta]:
        """Each SavedSliceMeta describes the slices for one tensor."""

    @property
    def versions(self) -> tensorflow.core.framework.versions_pb2.VersionDef:
        """Compatibility version of this checkpoint.  See core/public/version.h
        for version history.
        """

    def __init__(
        self,
        *,
        tensor: collections.abc.Iterable[global___SavedSliceMeta] | None = ...,
        versions: tensorflow.core.framework.versions_pb2.VersionDef | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["versions", b"versions"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["tensor", b"tensor", "versions", b"versions"]) -> None: ...

global___SavedTensorSliceMeta = SavedTensorSliceMeta

@typing.final
class SavedSlice(google.protobuf.message.Message):
    """Saved tensor slice: it stores the name of the tensors, the slice, and the
    raw data.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    SLICE_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Name of the tensor that this slice belongs to. This must be identical to
    the name used to encode the key for this record.
    """
    @property
    def slice(self) -> tensorflow.core.framework.tensor_slice_pb2.TensorSliceProto:
        """Extent of the slice.  Must have one entry for each of the dimension of the
        tensor that this slice belongs to.
        """

    @property
    def data(self) -> tensorflow.core.framework.tensor_pb2.TensorProto:
        """The raw data of the slice is stored as a TensorProto. Only raw data are
        stored (we don't fill in fields such as dtype or tensor_shape).
        """

    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        slice: tensorflow.core.framework.tensor_slice_pb2.TensorSliceProto | None = ...,
        data: tensorflow.core.framework.tensor_pb2.TensorProto | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["data", b"data", "slice", b"slice"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["data", b"data", "name", b"name", "slice", b"slice"]) -> None: ...

global___SavedSlice = SavedSlice

@typing.final
class SavedTensorSlices(google.protobuf.message.Message):
    """Each record in a v3 checkpoint file is a serialized SavedTensorSlices
    message.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    META_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    @property
    def meta(self) -> global___SavedTensorSliceMeta:
        """This is only present at the first item of each checkpoint file and serves
        as a table of contents, listing all the tensor slices saved in this file.
        """

    @property
    def data(self) -> global___SavedSlice:
        """This exists in all but the first item of each checkpoint file."""

    def __init__(self, *, meta: global___SavedTensorSliceMeta | None = ..., data: global___SavedSlice | None = ...) -> None: ...
    def HasField(self, field_name: typing.Literal["data", b"data", "meta", b"meta"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["data", b"data", "meta", b"meta"]) -> None: ...

global___SavedTensorSlices = SavedTensorSlices



import sys
from _typeshed import ReadableBuffer
from collections.abc import Sequence
from typing import Any, Final, Literal, NoReturn, final, overload

if sys.platform == "win32":
    ABOVE_NORMAL_PRIORITY_CLASS: Final = 0x8000
    BELOW_NORMAL_PRIORITY_CLASS: Final = 0x4000

    CREATE_BREAKAWAY_FROM_JOB: Final = 0x1000000
    CREATE_DEFAULT_ERROR_MODE: Final = 0x4000000
    CREATE_NO_WINDOW: Final = 0x8000000
    CREATE_NEW_CONSOLE: Final = 0x10
    CREATE_NEW_PROCESS_GROUP: Final = 0x200

    DETACHED_PROCESS: Final = 8
    DUPLICATE_CLOSE_SOURCE: Final = 1
    DUPLICATE_SAME_ACCESS: Final = 2

    ERROR_ALREADY_EXISTS: Final = 183
    ERROR_BROKEN_PIPE: Final = 109
    ERROR_IO_PENDING: Final = 997
    ERROR_MORE_DATA: Final = 234
    ERROR_NETNAME_DELETED: Final = 64
    ERROR_NO_DATA: Final = 232
    ERROR_NO_SYSTEM_RESOURCES: Final = 1450
    ERROR_OPERATION_ABORTED: Final = 995
    ERROR_PIPE_BUSY: Final = 231
    ERROR_PIPE_CONNECTED: Final = 535
    ERROR_SEM_TIMEOUT: Final = 121

    FILE_FLAG_FIRST_PIPE_INSTANCE: Final = 0x80000
    FILE_FLAG_OVERLAPPED: Final = 0x40000000

    FILE_GENERIC_READ: Final = 1179785
    FILE_GENERIC_WRITE: Final = 1179926

    FILE_MAP_ALL_ACCESS: Final = 983071
    FILE_MAP_COPY: Final = 1
    FILE_MAP_EXECUTE: Final = 32
    FILE_MAP_READ: Final = 4
    FILE_MAP_WRITE: Final = 2

    FILE_TYPE_CHAR: Final = 2
    FILE_TYPE_DISK: Final = 1
    FILE_TYPE_PIPE: Final = 3
    FILE_TYPE_REMOTE: Final = 32768
    FILE_TYPE_UNKNOWN: Final = 0

    GENERIC_READ: Final = 0x80000000
    GENERIC_WRITE: Final = 0x40000000
    HIGH_PRIORITY_CLASS: Final = 0x80
    INFINITE: Final = 0xFFFFFFFF
    # Ignore the Flake8 error -- flake8-pyi assumes
    # most numbers this long will be implementation details,
    # but here we can see that it's a power of 2
    INVALID_HANDLE_VALUE: Final = 0xFFFFFFFFFFFFFFFF  # noqa: Y054
    IDLE_PRIORITY_CLASS: Final = 0x40
    NORMAL_PRIORITY_CLASS: Final = 0x20
    REALTIME_PRIORITY_CLASS: Final = 0x100
    NMPWAIT_WAIT_FOREVER: Final = 0xFFFFFFFF

    MEM_COMMIT: Final = 0x1000
    MEM_FREE: Final = 0x10000
    MEM_IMAGE: Final = 0x1000000
    MEM_MAPPED: Final = 0x40000
    MEM_PRIVATE: Final = 0x20000
    MEM_RESERVE: Final = 0x2000

    NULL: Final = 0
    OPEN_EXISTING: Final = 3

    PIPE_ACCESS_DUPLEX: Final = 3
    PIPE_ACCESS_INBOUND: Final = 1
    PIPE_READMODE_MESSAGE: Final = 2
    PIPE_TYPE_MESSAGE: Final = 4
    PIPE_UNLIMITED_INSTANCES: Final = 255
    PIPE_WAIT: Final = 0

    PAGE_EXECUTE: Final = 0x10
    PAGE_EXECUTE_READ: Final = 0x20
    PAGE_EXECUTE_READWRITE: Final = 0x40
    PAGE_EXECUTE_WRITECOPY: Final = 0x80
    PAGE_GUARD: Final = 0x100
    PAGE_NOACCESS: Final = 0x1
    PAGE_NOCACHE: Final = 0x200
    PAGE_READONLY: Final = 0x2
    PAGE_READWRITE: Final = 0x4
    PAGE_WRITECOMBINE: Final = 0x400
    PAGE_WRITECOPY: Final = 0x8

    PROCESS_ALL_ACCESS: Final = 0x1FFFFF
    PROCESS_DUP_HANDLE: Final = 0x40

    SEC_COMMIT: Final = 0x8000000
    SEC_IMAGE: Final = 0x1000000
    SEC_LARGE_PAGES: Final = 0x80000000
    SEC_NOCACHE: Final = 0x10000000
    SEC_RESERVE: Final = 0x4000000
    SEC_WRITECOMBINE: Final = 0x40000000

    if sys.version_info >= (3, 13):
        STARTF_FORCEOFFFEEDBACK: Final = 0x80
        STARTF_FORCEONFEEDBACK: Final = 0x40
        STARTF_PREVENTPINNING: Final = 0x2000
        STARTF_RUNFULLSCREEN: Final = 0x20
        STARTF_TITLEISAPPID: Final = 0x1000
        STARTF_TITLEISLINKNAME: Final = 0x800
        STARTF_UNTRUSTEDSOURCE: Final = 0x8000
        STARTF_USECOUNTCHARS: Final = 0x8
        STARTF_USEFILLATTRIBUTE: Final = 0x10
        STARTF_USEHOTKEY: Final = 0x200
        STARTF_USEPOSITION: Final = 0x4
        STARTF_USESIZE: Final = 0x2

    STARTF_USESHOWWINDOW: Final = 0x1
    STARTF_USESTDHANDLES: Final = 0x100

    STD_ERROR_HANDLE: Final = 0xFFFFFFF4
    STD_OUTPUT_HANDLE: Final = 0xFFFFFFF5
    STD_INPUT_HANDLE: Final = 0xFFFFFFF6

    STILL_ACTIVE: Final = 259
    SW_HIDE: Final = 0
    SYNCHRONIZE: Final = 0x100000
    WAIT_ABANDONED_0: Final = 128
    WAIT_OBJECT_0: Final = 0
    WAIT_TIMEOUT: Final = 258

    if sys.version_info >= (3, 10):
        LOCALE_NAME_INVARIANT: Final[str]
        LOCALE_NAME_MAX_LENGTH: Final[int]
        LOCALE_NAME_SYSTEM_DEFAULT: Final[str]
        LOCALE_NAME_USER_DEFAULT: Final[str | None]

        LCMAP_FULLWIDTH: Final[int]
        LCMAP_HALFWIDTH: Final[int]
        LCMAP_HIRAGANA: Final[int]
        LCMAP_KATAKANA: Final[int]
        LCMAP_LINGUISTIC_CASING: Final[int]
        LCMAP_LOWERCASE: Final[int]
        LCMAP_SIMPLIFIED_CHINESE: Final[int]
        LCMAP_TITLECASE: Final[int]
        LCMAP_TRADITIONAL_CHINESE: Final[int]
        LCMAP_UPPERCASE: Final[int]

    if sys.version_info >= (3, 12):
        COPYFILE2_CALLBACK_CHUNK_STARTED: Final = 1
        COPYFILE2_CALLBACK_CHUNK_FINISHED: Final = 2
        COPYFILE2_CALLBACK_STREAM_STARTED: Final = 3
        COPYFILE2_CALLBACK_STREAM_FINISHED: Final = 4
        COPYFILE2_CALLBACK_POLL_CONTINUE: Final = 5
        COPYFILE2_CALLBACK_ERROR: Final = 6

        COPYFILE2_PROGRESS_CONTINUE: Final = 0
        COPYFILE2_PROGRESS_CANCEL: Final = 1
        COPYFILE2_PROGRESS_STOP: Final = 2
        COPYFILE2_PROGRESS_QUIET: Final = 3
        COPYFILE2_PROGRESS_PAUSE: Final = 4

        COPY_FILE_FAIL_IF_EXISTS: Final = 0x1
        COPY_FILE_RESTARTABLE: Final = 0x2
        COPY_FILE_OPEN_SOURCE_FOR_WRITE: Final = 0x4
        COPY_FILE_ALLOW_DECRYPTED_DESTINATION: Final = 0x8
        COPY_FILE_COPY_SYMLINK: Final = 0x800
        COPY_FILE_NO_BUFFERING: Final = 0x1000
        COPY_FILE_REQUEST_SECURITY_PRIVILEGES: Final = 0x2000
        COPY_FILE_RESUME_FROM_PAUSE: Final = 0x4000
        COPY_FILE_NO_OFFLOAD: Final = 0x40000
        COPY_FILE_REQUEST_COMPRESSED_TRAFFIC: Final = 0x10000000

        ERROR_ACCESS_DENIED: Final = 5
        ERROR_PRIVILEGE_NOT_HELD: Final = 1314

    if sys.version_info >= (3, 14):
        COPY_FILE_DIRECTORY: Final = 0x00000080

    def CloseHandle(handle: int, /) -> None: ...
    @overload
    def ConnectNamedPipe(handle: int, overlapped: Literal[True]) -> Overlapped: ...
    @overload
    def ConnectNamedPipe(handle: int, overlapped: Literal[False] = False) -> None: ...
    @overload
    def ConnectNamedPipe(handle: int, overlapped: bool) -> Overlapped | None: ...
    def CreateFile(
        file_name: str,
        desired_access: int,
        share_mode: int,
        security_attributes: int,
        creation_disposition: int,
        flags_and_attributes: int,
        template_file: int,
        /,
    ) -> int: ...
    def CreateJunction(src_path: str, dst_path: str, /) -> None: ...
    def CreateNamedPipe(
        name: str,
        open_mode: int,
        pipe_mode: int,
        max_instances: int,
        out_buffer_size: int,
        in_buffer_size: int,
        default_timeout: int,
        security_attributes: int,
        /,
    ) -> int: ...
    def CreatePipe(pipe_attrs: Any, size: int, /) -> tuple[int, int]: ...
    def CreateProcess(
        application_name: str | None,
        command_line: str | None,
        proc_attrs: Any,
        thread_attrs: Any,
        inherit_handles: bool,
        creation_flags: int,
        env_mapping: dict[str, str],
        current_directory: str | None,
        startup_info: Any,
        /,
    ) -> tuple[int, int, int, int]: ...
    def DuplicateHandle(
        source_process_handle: int,
        source_handle: int,
        target_process_handle: int,
        desired_access: int,
        inherit_handle: bool,
        options: int = 0,
        /,
    ) -> int: ...
    def ExitProcess(ExitCode: int, /) -> NoReturn: ...
    def GetACP() -> int: ...
    def GetFileType(handle: int) -> int: ...
    def GetCurrentProcess() -> int: ...
    def GetExitCodeProcess(process: int, /) -> int: ...
    def GetLastError() -> int: ...
    def GetModuleFileName(module_handle: int, /) -> str: ...
    def GetStdHandle(std_handle: int, /) -> int: ...
    def GetVersion() -> int: ...
    def OpenProcess(desired_access: int, inherit_handle: bool, process_id: int, /) -> int: ...
    def PeekNamedPipe(handle: int, size: int = 0, /) -> tuple[int, int] | tuple[bytes, int, int]: ...
    if sys.version_info >= (3, 10):
        def LCMapStringEx(locale: str, flags: int, src: str) -> str: ...
        def UnmapViewOfFile(address: int, /) -> None: ...

    @overload
    def ReadFile(handle: int, size: int, overlapped: Literal[True]) -> tuple[Overlapped, int]: ...
    @overload
    def ReadFile(handle: int, size: int, overlapped: Literal[False] = False) -> tuple[bytes, int]: ...
    @overload
    def ReadFile(handle: int, size: int, overlapped: int | bool) -> tuple[Any, int]: ...
    def SetNamedPipeHandleState(
        named_pipe: int, mode: int | None, max_collection_count: int | None, collect_data_timeout: int | None, /
    ) -> None: ...
    def TerminateProcess(handle: int, exit_code: int, /) -> None: ...
    def WaitForMultipleObjects(handle_seq: Sequence[int], wait_flag: bool, milliseconds: int = 0xFFFFFFFF, /) -> int: ...
    def WaitForSingleObject(handle: int, milliseconds: int, /) -> int: ...
    def WaitNamedPipe(name: str, timeout: int, /) -> None: ...
    @overload
    def WriteFile(handle: int, buffer: ReadableBuffer, overlapped: Literal[True]) -> tuple[Overlapped, int]: ...
    @overload
    def WriteFile(handle: int, buffer: ReadableBuffer, overlapped: Literal[False] = False) -> tuple[int, int]: ...
    @overload
    def WriteFile(handle: int, buffer: ReadableBuffer, overlapped: int | bool) -> tuple[Any, int]: ...
    @final
    class Overlapped:
        event: int
        def GetOverlappedResult(self, wait: bool, /) -> tuple[int, int]: ...
        def cancel(self) -> None: ...
        def getbuffer(self) -> bytes | None: ...

    if sys.version_info >= (3, 13):
        def BatchedWaitForMultipleObjects(
            handle_seq: Sequence[int], wait_all: bool, milliseconds: int = 0xFFFFFFFF
        ) -> list[int]: ...
        def CreateEventW(security_attributes: int, manual_reset: bool, initial_state: bool, name: str | None) -> int: ...
        def CreateMutexW(security_attributes: int, initial_owner: bool, name: str) -> int: ...
        def GetLongPathName(path: str) -> str: ...
        def GetShortPathName(path: str) -> str: ...
        def OpenEventW(desired_access: int, inherit_handle: bool, name: str) -> int: ...
        def OpenMutexW(desired_access: int, inherit_handle: bool, name: str) -> int: ...
        def ReleaseMutex(mutex: int) -> None: ...
        def ResetEvent(event: int) -> None: ...
        def SetEvent(event: int) -> None: ...

    if sys.version_info >= (3, 12):
        def CopyFile2(existing_file_name: str, new_file_name: str, flags: int, progress_routine: int | None = None) -> int: ...
        def NeedCurrentDirectoryForExePath(exe_name: str, /) -> bool: ...



import datetime

from ..core import WesternCalendar

class Sweden(WesternCalendar):
    def get_midsummer_eve(self, year: int) -> datetime.date: ...
    def get_midsummer_day(self, year: int) -> datetime.date: ...
    def get_variable_all_saints(self, year: int) -> datetime.date: ...



from collections.abc import Callable
from typing import TypeVar, overload
from typing_extensions import TypeAlias

_AnyCallable: TypeAlias = Callable[..., object]
_Func = TypeVar("_Func", bound=_AnyCallable)

# We (ab)use this decorator to also copy the signature of the source function (overload 1)
# The advantages are:
# - avoid copying all parameters and types manually while conserving type safety
# - signature properly handeled in IDEs (at least with Pylance)
# - docstring from the original function properly displayed (at least with Pylance)
# Using the other overloads returns the signature of the decorated function instead
@overload
def doc(func: _Func, /, **params: object) -> Callable[[_AnyCallable], _Func]: ...
@overload
def doc(docstring: str, /, *docstrings: str | _AnyCallable, **params: object) -> Callable[[_Func], _Func]: ...
@overload
def doc(
    docstring1: str | _AnyCallable, docstring2: str | _AnyCallable, /, *docstrings: str | _AnyCallable, **params: object
) -> Callable[[_Func], _Func]: ...



from logging import Logger

from .base import BaseEndpoint as BaseEndpoint

log: Logger

class AccessTokenEndpoint(BaseEndpoint):
    def create_access_token(self, request, credentials): ...
    def create_access_token_response(self, uri, http_method: str = "GET", body=None, headers=None, credentials=None): ...
    def validate_access_token_request(self, request): ...



from typing import Final

from .api import FFI as FFI
from .error import (
    CDefError as CDefError,
    FFIError as FFIError,
    PkgConfigError as PkgConfigError,
    VerificationError as VerificationError,
    VerificationMissing as VerificationMissing,
)

__all__ = ["FFI", "VerificationError", "VerificationMissing", "CDefError", "FFIError"]
__version__: Final[str]
__version_info__: Final[tuple[int, int, int]]
__version_verifier_modules__: Final[str]



import socket

from gunicorn.http import Request
from gunicorn.workers.base import Worker

from .._types import _AddressType

class StopWaiting(Exception): ...

class SyncWorker(Worker):
    def accept(self, listener: socket.socket) -> None: ...
    def wait(self, timeout: int) -> list[socket.socket | int] | None: ...
    def is_parent_alive(self) -> bool: ...
    def run_for_one(self, timeout: int) -> None: ...
    def run_for_multiple(self, timeout: int) -> None: ...
    def run(self) -> None: ...
    def handle(self, listener: socket.socket, client: socket.socket, addr: _AddressType) -> None: ...
    def handle_request(self, listener: socket.socket, req: Request, client: socket.socket, addr: tuple[str, int]) -> bool: ...
    def handle_error(self, req: Request | None, client: socket.socket, addr: _AddressType, exc: BaseException) -> None: ...



from _typeshed import Unused
from collections.abc import Generator
from typing import Final, Literal, NoReturn, overload

EPOCH: Final = 584282.5
HAAB: Final[list[str]]
HAAB_TRANSLATIONS: Final[list[str]]
TZOLKIN: Final[list[str]]
TZOLKIN_TRANSLATIONS: Final[list[str]]

def to_jd(baktun: int, katun: int, tun: int, uinal: int, kin: int) -> float: ...
def from_jd(jd: float) -> tuple[int, int, int, int, int]: ...
def to_gregorian(baktun: int, katun: int, tun: int, uinal: int, kin: int) -> tuple[int, int, int]: ...
def from_gregorian(year: int, month: int, day: int) -> tuple[int, int, int, int, int]: ...
def to_haab(jd: float) -> tuple[int, str]: ...
def to_tzolkin(jd: float) -> tuple[int, str]: ...
def lc_to_haab(baktun: int, katun: int, tun: int, uinal: int, kin: int) -> tuple[int, str]: ...
def lc_to_tzolkin(baktun: int, katun: int, tun: int, uinal: int, kin: int) -> tuple[int, str]: ...
def lc_to_haab_tzolkin(baktun: int, katun: int, tun: int, uinal: int, kin: int) -> str: ...
def translate_haab(h: str) -> str | None: ...
def translate_tzolkin(tz: str) -> str | None: ...
def tzolkin_generator(number: int | None = None, name: str | None = None) -> Generator[tuple[int, str]]: ...
def longcount_generator(
    baktun: int, katun: int, tun: int, uinal: int, kin: int
) -> Generator[tuple[int, int, int, int, int], None, NoReturn]: ...
def next_haab(month: str, jd: float) -> float: ...
def next_tzolkin(tzolkin: tuple[int, str], jd: float) -> float: ...
def next_tzolkin_haab(tzolkin: tuple[int, str], haab: tuple[int, str], jd: float) -> float: ...
def month_length(month: str) -> Literal[5, 20]: ...
@overload
def haab_monthcalendar(
    baktun: int, katun: int, tun: int, uinal: int, kin: int, jdc: None = None
) -> list[list[tuple[int | None, tuple[int, str] | None, tuple[int, int, int, int, int] | None]]]: ...
@overload
def haab_monthcalendar(
    baktun: Unused = None, katun: Unused = None, tun: Unused = None, uinal: Unused = None, kin: Unused = None, jdc: float = ...
) -> list[list[tuple[int | None, tuple[int, str] | None, tuple[int, int, int, int, int] | None]]]: ...
def haab_monthcalendar_prospective(
    haabmonth: str, jdc: float
) -> list[list[tuple[int | None, tuple[int, str] | None, tuple[int, int, int, int, int] | None]]]: ...



def system_path_separator() -> str: ...
def XDG_DATA_HOME() -> str: ...
def XDG_CONFIG_HOME() -> str: ...
def XDG_CACHE_HOME() -> str: ...
def XDG_RUNTIME_DIR() -> str: ...
def XDG_DATA_DIRS() -> str: ...
def XDG_CONFIG_DIRS() -> str: ...



from collections.abc import Collection
from typing import overload
from typing_extensions import Self

from .._typing import OptGeoArrayLike
from .base import BaseMultipartGeometry, GeometrySequence, _GeoT_co

class GeometryCollection(BaseMultipartGeometry[_GeoT_co]):
    # Overloads of __new__ are used because mypy is unable to narrow the typevar otherwise
    __slots__: list[str] = []
    @overload
    def __new__(
        self, geoms: BaseMultipartGeometry[_GeoT_co] | GeometrySequence[BaseMultipartGeometry[_GeoT_co]] | Collection[_GeoT_co]
    ) -> Self: ...
    @overload
    def __new__(self, geoms: OptGeoArrayLike = None) -> Self: ...
    # more precise base overrides
    @property
    def boundary(self) -> None: ...



from braintree.add_on_gateway import AddOnGateway
from braintree.address_gateway import AddressGateway
from braintree.apple_pay_gateway import ApplePayGateway
from braintree.bank_account_instant_verification_gateway import BankAccountInstantVerificationGateway
from braintree.client_token_gateway import ClientTokenGateway
from braintree.configuration import Configuration
from braintree.credit_card_gateway import CreditCardGateway
from braintree.credit_card_verification_gateway import CreditCardVerificationGateway
from braintree.customer_gateway import CustomerGateway
from braintree.discount_gateway import DiscountGateway
from braintree.dispute_gateway import DisputeGateway
from braintree.document_upload_gateway import DocumentUploadGateway
from braintree.exchange_rate_quote_gateway import ExchangeRateQuoteGateway
from braintree.merchant_account_gateway import MerchantAccountGateway
from braintree.merchant_gateway import MerchantGateway
from braintree.oauth_gateway import OAuthGateway
from braintree.payment_method_gateway import PaymentMethodGateway
from braintree.payment_method_nonce_gateway import PaymentMethodNonceGateway
from braintree.paypal_account_gateway import PayPalAccountGateway
from braintree.paypal_payment_resource_gateway import PayPalPaymentResourceGateway
from braintree.plan_gateway import PlanGateway
from braintree.sepa_direct_debit_account_gateway import SepaDirectDebitAccountGateway
from braintree.settlement_batch_summary_gateway import SettlementBatchSummaryGateway
from braintree.subscription_gateway import SubscriptionGateway
from braintree.testing_gateway import TestingGateway
from braintree.transaction_gateway import TransactionGateway
from braintree.transaction_line_item_gateway import TransactionLineItemGateway
from braintree.us_bank_account_gateway import UsBankAccountGateway
from braintree.us_bank_account_verification_gateway import UsBankAccountVerificationGateway
from braintree.util.graphql_client import GraphQLClient
from braintree.webhook_notification_gateway import WebhookNotificationGateway
from braintree.webhook_testing_gateway import WebhookTestingGateway

class BraintreeGateway:
    config: Configuration
    add_on: AddOnGateway
    address: AddressGateway
    apple_pay: ApplePayGateway
    bank_account_instant_verification: BankAccountInstantVerificationGateway
    client_token: ClientTokenGateway
    credit_card: CreditCardGateway
    customer: CustomerGateway
    discount: DiscountGateway
    dispute: DisputeGateway
    document_upload: DocumentUploadGateway
    exchange_rate_quote: ExchangeRateQuoteGateway
    graphql_client: GraphQLClient
    merchant: MerchantGateway
    merchant_account: MerchantAccountGateway
    oauth: OAuthGateway
    payment_method: PaymentMethodGateway
    payment_method_nonce: PaymentMethodNonceGateway
    paypal_account: PayPalAccountGateway
    paypal_payment_resource: PayPalPaymentResourceGateway
    plan: PlanGateway
    sepa_direct_debit_account: SepaDirectDebitAccountGateway
    settlement_batch_summary: SettlementBatchSummaryGateway
    subscription: SubscriptionGateway
    testing: TestingGateway
    transaction: TransactionGateway
    transaction_line_item: TransactionLineItemGateway
    us_bank_account: UsBankAccountGateway
    us_bank_account_verification: UsBankAccountVerificationGateway
    verification: CreditCardVerificationGateway
    webhook_notification: WebhookNotificationGateway
    webhook_testing: WebhookTestingGateway
    def __init__(self, config=None, **kwargs) -> None: ...



from _typeshed import Incomplete
from collections.abc import Callable
from typing_extensions import Never

from networkx.algorithms.flow import edmonds_karp
from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["minimum_st_node_cut", "minimum_node_cut", "minimum_st_edge_cut", "minimum_edge_cut"]
default_flow_func = edmonds_karp

@_dispatchable
def minimum_st_edge_cut(
    G: Graph[_Node],
    s: _Node,
    t: _Node,
    flow_func: Callable[..., Incomplete] | None = None,
    auxiliary: DiGraph[_Node] | None = None,
    residual: DiGraph[_Node] | None = None,
) -> set[tuple[Incomplete, Incomplete]]: ...
@_dispatchable
def minimum_st_node_cut(
    G: Graph[_Node],
    s: _Node,
    t: _Node,
    flow_func: Callable[..., Incomplete] | None = None,
    auxiliary: DiGraph[_Node] | None = None,
    residual: DiGraph[_Node] | None = None,
) -> dict[Never, Never] | set[Incomplete]: ...
@_dispatchable
def minimum_node_cut(
    G: Graph[_Node], s: _Node | None = None, t: _Node | None = None, flow_func: Callable[..., Incomplete] | None = None
) -> dict[Never, Never] | set[Incomplete]: ...
@_dispatchable
def minimum_edge_cut(
    G: Graph[_Node], s: _Node | None = None, t: _Node | None = None, flow_func: Callable[..., Incomplete] | None = None
) -> set[Incomplete]: ...



from _typeshed import FileDescriptorOrPath, SupportsWrite
from collections.abc import Callable
from io import BufferedReader
from multiprocessing import Process
from multiprocessing.queues import Queue
from multiprocessing.synchronize import Event
from signal import Signals
from types import FrameType, TracebackType
from typing import IO, Any, NoReturn
from typing_extensions import Self

__version__: str
aliases: dict[str, Any]
DEFAULT_TEXT_ENCODING: str
GRACEFUL_SHUTDOWN_SIGNAL: Signals
TERMINATION_DELAY: float
PARTIAL_DEFAULT: bool
STDOUT_FD: int
STDERR_FD: int

def enable_old_api() -> None: ...
def create_proxy_method(name: str) -> Callable[..., Any]: ...

class MultiProcessHelper:
    processes: list[Process]
    def __init__(self) -> None: ...
    def start_child(self, target: Callable[[Event], Any]) -> None: ...
    def stop_children(self) -> None: ...
    def wait_for_children(self) -> None: ...
    def enable_graceful_shutdown(self) -> None: ...
    def raise_shutdown_request(self, signum: int, frame: FrameType | None) -> NoReturn: ...

class CaptureOutput(MultiProcessHelper):
    chunk_size: int
    encoding: str
    merged: bool
    relay: bool
    termination_delay: float
    pseudo_terminals: list[PseudoTerminal]
    streams: list[tuple[int, Stream]]
    stdout_stream: Stream
    stderr_stream: Stream
    output: PseudoTerminal
    output_queue: Queue[tuple[Any, bytes]]
    stdout: PseudoTerminal
    stderr: PseudoTerminal
    def __init__(
        self, merged: bool = True, encoding: str = ..., termination_delay: float = ..., chunk_size: int = 1024, relay: bool = True
    ) -> None: ...
    def initialize_stream(self, file_obj: IO[str], expected_fd: int) -> Stream: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None = None,
        exc_val: BaseException | None = None,
        exc_tb: TracebackType | None = None,
    ) -> bool | None: ...
    @property
    def is_capturing(self) -> bool: ...
    def start_capture(self) -> None: ...
    def finish_capture(self) -> None: ...
    def allocate_pty(
        self, relay_fd: int | None = None, output_queue: Queue[tuple[Any, bytes]] | None = None, queue_token: Any | None = None
    ) -> PseudoTerminal: ...
    def merge_loop(self, started_event: Event) -> None: ...
    def get_handle(self, partial: bool = ...) -> BufferedReader: ...
    def get_bytes(self, partial: bool = ...) -> bytes: ...
    def get_lines(self, interpreted: bool = True, partial: bool = ...) -> list[str]: ...
    def get_text(self, interpreted: bool = ..., partial: bool = ...) -> str: ...
    def save_to_handle(self, handle: SupportsWrite[bytes], partial: bool = ...) -> None: ...
    def save_to_path(self, filename: FileDescriptorOrPath, partial: bool = ...) -> None: ...

class OutputBuffer:
    fd: int
    buffer: bytes
    def __init__(self, fd: int) -> None: ...
    def add(self, output: bytes) -> None: ...
    def flush(self) -> None: ...

class PseudoTerminal(MultiProcessHelper):
    encoding: str
    termination_delay: float
    chunk_size: int
    relay_fd: int | None
    output_queue: Queue[tuple[Any, bytes]] | None
    queue_token: Any | None
    streams: list[Stream]
    master_fd: int
    slave_fd: int
    output_fd: int
    output_handle: BufferedReader
    def __init__(
        self,
        encoding: str,
        termination_delay: float,
        chunk_size: int,
        relay_fd: int | None,
        output_queue: Queue[tuple[int, bytes]] | None,
        queue_token: Any | None,
    ) -> None: ...
    def attach(self, stream: Stream) -> None: ...
    def start_capture(self) -> None: ...
    def finish_capture(self) -> None: ...
    def close_pseudo_terminal(self) -> None: ...
    def restore_streams(self) -> None: ...
    def get_handle(self, partial: bool = ...) -> BufferedReader: ...
    def get_bytes(self, partial: bool = ...) -> bytes: ...
    def get_lines(self, interpreted: bool = True, partial: bool = ...) -> list[str]: ...
    def get_text(self, interpreted: bool = ..., partial: bool = ...) -> str: ...
    def save_to_handle(self, handle: SupportsWrite[bytes], partial: bool = ...) -> None: ...
    def save_to_path(self, filename: FileDescriptorOrPath, partial: bool = ...) -> None: ...
    def capture_loop(self, started_event: Event) -> None: ...

class Stream:
    fd: int
    original_fd: int
    is_redirected: bool
    def __init__(self, fd: int) -> None: ...
    def redirect(self, target_fd: int) -> None: ...
    def restore(self) -> None: ...

class ShutdownRequested(Exception): ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import typing

import google.protobuf.descriptor
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class RPCOptions(google.protobuf.message.Message):
    """RPC options for distributed runtime."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    USE_RPC_FOR_INPROCESS_MASTER_FIELD_NUMBER: builtins.int
    COMPRESSION_ALGORITHM_FIELD_NUMBER: builtins.int
    COMPRESSION_LEVEL_FIELD_NUMBER: builtins.int
    CACHE_RPC_RESPONSE_FIELD_NUMBER: builtins.int
    DISABLE_SESSION_CONNECTION_SHARING_FIELD_NUMBER: builtins.int
    NUM_CHANNELS_PER_TARGET_FIELD_NUMBER: builtins.int
    use_rpc_for_inprocess_master: builtins.bool
    """If true, always use RPC to contact the session target.

    If false (the default option), TensorFlow may use an optimized
    transport for client-master communication that avoids the RPC
    stack. This option is primarily for used testing the RPC stack.
    """
    compression_algorithm: builtins.str
    """The compression algorithm to be used. One of "deflate", "gzip"."""
    compression_level: builtins.int
    """If compression_algorithm is set, the compression level to be used.
    From 0 (no compression), up to 3.
    """
    cache_rpc_response: builtins.bool
    """Setting cache_rpc_response to true will enable sender side caching of
    response for RecvTensorAsync and RecvBufAsync to allow receiver to retry
    requests . This is only necessary when the network fabric is experiencing a
    significant error rate.  Without it we'll fail a step on an network error,
    while with it we'll be able to complete long steps (like complex
    initializations) in the face of some network errors during RecvTensor.
    """
    disable_session_connection_sharing: builtins.bool
    """Disables TCP connection sharing when opening a new RPC channel."""
    num_channels_per_target: builtins.int
    """Setting num_channels_per_target > 0 allows uses of multiple channels to
    communicate to the same target. This can be used to improve the aggregate
    throughput on high speed links (e.g 100G) where single connection is not
    sufficient to maximize link utilization. Note that a single RPC only goes
    on a single channel, this only helps in situations where there are multiple
    transfers to the same target overlapping in time.
    """
    def __init__(
        self,
        *,
        use_rpc_for_inprocess_master: builtins.bool | None = ...,
        compression_algorithm: builtins.str | None = ...,
        compression_level: builtins.int | None = ...,
        cache_rpc_response: builtins.bool | None = ...,
        disable_session_connection_sharing: builtins.bool | None = ...,
        num_channels_per_target: builtins.int | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "cache_rpc_response",
            b"cache_rpc_response",
            "compression_algorithm",
            b"compression_algorithm",
            "compression_level",
            b"compression_level",
            "disable_session_connection_sharing",
            b"disable_session_connection_sharing",
            "num_channels_per_target",
            b"num_channels_per_target",
            "use_rpc_for_inprocess_master",
            b"use_rpc_for_inprocess_master",
        ],
    ) -> None: ...

global___RPCOptions = RPCOptions



from collections.abc import Iterable
from logging import Logger

from oauthlib.common import Request
from oauthlib.oauth2.rfc6749.grant_types.base import _AuthValidator, _TokenValidator
from oauthlib.oauth2.rfc6749.grant_types.implicit import ImplicitGrant as OAuth2ImplicitGrant
from oauthlib.oauth2.rfc6749.request_validator import RequestValidator as OAuth2RequestValidator

from .base import GrantTypeBase

log: Logger

class ImplicitGrant(GrantTypeBase):
    proxy_target: OAuth2ImplicitGrant
    def __init__(
        self,
        request_validator: OAuth2RequestValidator | None = None,
        *,
        post_auth: Iterable[_AuthValidator] | None = None,
        post_token: Iterable[_TokenValidator] | None = None,
        pre_auth: Iterable[_AuthValidator] | None = None,
        pre_token: Iterable[_TokenValidator] | None = None,
        **kwargs,
    ) -> None: ...
    def add_id_token(self, token, token_handler, request: Request): ...  # type: ignore[override]
    def openid_authorization_validator(self, request: Request): ...



from collections.abc import Iterable, Mapping
from types import TracebackType
from typing import Literal, NamedTuple
from typing_extensions import Self, TypeAlias

from tensorflow._aliases import DTypeLike, ShapeLike, TensorCompatible, TensorLike
from tensorflow.io import gfile as gfile

_FeatureSpecs: TypeAlias = Mapping[str, FixedLenFeature | FixedLenSequenceFeature | VarLenFeature | RaggedFeature | SparseFeature]

_CompressionTypes: TypeAlias = Literal["ZLIB", "GZIP", "AUTO", "", 0, 1, 2] | None
_CompressionLevels: TypeAlias = Literal[0, 1, 2, 3, 4, 5, 6, 7, 8, 9] | None
_MemoryLevels: TypeAlias = Literal[1, 2, 3, 4, 5, 6, 7, 8, 9] | None

class TFRecordOptions:
    compression_type: _CompressionTypes | TFRecordOptions
    flush_mode: int | None  # The exact values allowed comes from zlib
    input_buffer_size: int | None
    output_buffer_size: int | None
    window_bits: int | None
    compression_level: _CompressionLevels
    compression_method: str | None
    mem_level: _MemoryLevels
    compression_strategy: int | None  # The exact values allowed comes from zlib

    def __init__(
        self,
        compression_type: _CompressionTypes | TFRecordOptions = None,
        flush_mode: int | None = None,
        input_buffer_size: int | None = None,
        output_buffer_size: int | None = None,
        window_bits: int | None = None,
        compression_level: _CompressionLevels = None,
        compression_method: str | None = None,
        mem_level: _MemoryLevels = None,
        compression_strategy: int | None = None,
    ) -> None: ...
    @classmethod
    def get_compression_type_string(cls, options: _CompressionTypes | TFRecordOptions) -> str: ...

class TFRecordWriter:
    def __init__(self, path: str, options: _CompressionTypes | TFRecordOptions | None = None) -> None: ...
    def write(self, record: bytes) -> None: ...
    def flush(self) -> None: ...
    def close(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

class FixedLenFeature(NamedTuple):
    shape: ShapeLike
    dtype: DTypeLike
    default_value: TensorCompatible | None = None

class FixedLenSequenceFeature(NamedTuple):
    shape: ShapeLike
    dtype: DTypeLike
    allow_missing: bool = False
    default_value: TensorCompatible | None = None

class VarLenFeature(NamedTuple):
    dtype: DTypeLike

class SparseFeature(NamedTuple):
    index_key: str | list[str]
    value_key: str
    dtype: DTypeLike
    size: int | list[int]
    already_sorted: bool = False

class RaggedFeature(NamedTuple):
    # Mypy doesn't support nested NamedTuples, but at runtime they actually do use
    # nested collections.namedtuple.
    class RowSplits(NamedTuple):  # type: ignore[misc]
        key: str

    class RowLengths(NamedTuple):  # type: ignore[misc]
        key: str

    class RowStarts(NamedTuple):  # type: ignore[misc]
        key: str

    class RowLimits(NamedTuple):  # type: ignore[misc]
        key: str

    class ValueRowIds(NamedTuple):  # type: ignore[misc]
        key: str

    class UniformRowLength(NamedTuple):  # type: ignore[misc]
        length: int

    dtype: DTypeLike
    value_key: str | None = None
    partitions: tuple[  # type: ignore[name-defined]
        RowSplits | RowLengths | RowStarts | RowLimits | ValueRowIds | UniformRowLength, ...
    ] = ()
    row_splits_dtype: DTypeLike = ...
    validate: bool = False

def parse_example(
    serialized: TensorCompatible, features: _FeatureSpecs, example_names: Iterable[str] | None = None, name: str | None = None
) -> dict[str, TensorLike]: ...
def __getattr__(name: str): ...  # incomplete module



from typing import Any

from django.core.exceptions import FieldError
from django.db import models

class FieldLookupError(FieldError):
    # Field type params are runtime-determined
    def __init__(self, model_field: models.Field[Any, Any], lookup_expr: str) -> None: ...



from types import TracebackType

from win32com.server.exception import COMException
from win32comext.axscript.client.debug import DebugManager
from win32comext.axscript.client.framework import AXScriptCodeBlock, COMScript
from win32comext.axscript.server.axsite import AXSite

debugging: int

def FormatForAX(text: str) -> str: ...
def ExpandTabs(text: str) -> str: ...
def AddCR(text: str) -> str: ...

class IActiveScriptError:
    def GetSourceLineText(self) -> str | None: ...
    def GetSourcePosition(self) -> tuple[int, int, int]: ...
    def GetExceptionInfo(self) -> AXScriptException: ...

class AXScriptException(COMException):
    sourceContext: int
    startLineNo: int
    linetext: str
    def __init__(
        self,
        site: COMScript,
        codeBlock: AXScriptCodeBlock | None,
        exc_type: None = None,
        exc_value: BaseException | None = None,
        exc_traceback: None = None,
    ) -> None: ...
    def ExtractTracebackInfo(self, tb: TracebackType, site: COMScript) -> tuple[str, int, str, str | None]: ...

def ProcessAXScriptException(
    scriptingSite: AXSite, debugManager: DebugManager, exceptionInstance: AXScriptException
) -> None | COMException | AXScriptException: ...



import random
from abc import abstractmethod
from collections.abc import Callable, Iterator, MutableMapping, Sequence
from typing import Any, Final, Literal, overload
from typing_extensions import Self, TypeAlias

class SequenceGenerator:
    length: int | None
    requested_entropy: str
    rng: random.Random
    @property
    @abstractmethod
    def symbol_count(self) -> int: ...
    def __init__(self, entropy: int | None = None, length: int | None = None, rng: random.Random | None = None) -> None: ...
    @property
    def entropy_per_symbol(self) -> float: ...
    @property
    def entropy(self) -> float: ...
    def __next__(self) -> str: ...
    @overload
    def __call__(self, returns: None = None) -> str: ...
    @overload
    def __call__(self, returns: int) -> list[str]: ...
    @overload
    def __call__(self, returns: Callable[[Any], Iterator[Any]]) -> Iterator[str]: ...  # "returns" must be the "iter" builtin
    def __iter__(self) -> Self: ...

_Charset: TypeAlias = Literal["ascii_72", "ascii_62", "ascii_50", "hex"]
default_charsets: Final[dict[_Charset, str]]

class WordGenerator(SequenceGenerator):
    charset: _Charset
    chars: str | bytes | None
    def __init__(
        self,
        chars: str | bytes | None = None,
        charset: _Charset | None = None,
        *,
        entropy: int | None = None,
        length: int | None = None,
        rng: random.Random | None = None,
    ) -> None: ...
    @property
    def symbol_count(self) -> int: ...

@overload
def genword(
    entropy: int | None = None,
    length: int | None = None,
    returns: None = None,
    *,
    chars: str | None = None,
    charset: _Charset | None = None,
    rng: random.Random | None = None,
) -> str: ...
@overload
def genword(
    returns: int,
    entropy: int | None = None,
    length: int | None = None,
    *,
    chars: str | None = None,
    charset: _Charset | None = None,
    rng: random.Random | None = None,
) -> list[str]: ...
@overload
def genword(
    returns: Callable[[Any], Iterator[Any]],
    entropy: int | None = None,
    length: int | None = None,
    *,
    chars: str | None = None,
    charset: _Charset | None = None,
    rng: random.Random | None = None,
) -> Iterator[str]: ...

class WordsetDict(MutableMapping[Any, Any]):
    paths: dict[str, str] | None
    def __init__(self, *args, **kwds) -> None: ...
    def __getitem__(self, key: str) -> tuple[str | bytes, ...]: ...
    def set_path(self, key: str, path: str) -> None: ...
    def __setitem__(self, key: str, value: tuple[str | bytes, ...]) -> None: ...
    def __delitem__(self, key: str) -> None: ...
    def __iter__(self) -> Iterator[str]: ...
    def __len__(self) -> int: ...
    def __contains__(self, key: object) -> bool: ...

default_wordsets: WordsetDict
_Wordset: TypeAlias = Literal["eff_long", "eff_short", "eff_prefixed", "bip39"]

class PhraseGenerator(SequenceGenerator):
    wordset: _Wordset
    words: Sequence[str | bytes] | None
    sep: str | None
    def __init__(
        self,
        wordset: _Wordset | None = None,
        words: Sequence[str | bytes] | None = None,
        sep: str | bytes | None = None,
        *,
        entropy: int | None = None,
        length: int | None = None,
        rng: random.Random | None = None,
    ) -> None: ...
    @property
    def symbol_count(self) -> int: ...

@overload
def genphrase(
    entropy: int | None = None,
    length: int | None = None,
    returns: None = None,
    *,
    wordset: _Wordset | None = None,
    words: Sequence[str | bytes] | None = None,
    sep: str | bytes | None = None,
    rng: random.Random | None = None,
) -> str: ...
@overload
def genphrase(
    returns: int,
    entropy: int | None = None,
    length: int | None = None,
    *,
    wordset: _Wordset | None = None,
    words: Sequence[str | bytes] | None = None,
    sep: str | bytes | None = None,
    rng: random.Random | None = None,
) -> list[str]: ...
@overload
def genphrase(
    returns: Callable[[Any], Iterator[Any]],
    entropy: int | None = None,
    length: int | None = None,
    *,
    wordset: _Wordset | None = None,
    words: Sequence[str | bytes] | None = None,
    sep: str | bytes | None = None,
    rng: random.Random | None = None,
) -> Iterator[str]: ...

__all__ = ["genword", "default_charsets", "genphrase", "default_wordsets"]



__all__ = ["TZProvider"]

import datetime
from abc import ABC, abstractmethod

from dateutil.rrule import rrule

from ..prop import vRecur

class TZProvider(ABC):
    @property
    @abstractmethod
    def name(self) -> str: ...
    @abstractmethod
    def localize_utc(self, dt: datetime.datetime) -> datetime.datetime: ...
    @abstractmethod
    def localize(self, dt: datetime.datetime, tz: datetime.tzinfo) -> datetime.datetime: ...
    @abstractmethod
    def knows_timezone_id(self, id: str) -> bool: ...
    @abstractmethod
    def fix_rrule_until(self, rrule: rrule, ical_rrule: vRecur) -> None: ...
    @abstractmethod
    def create_timezone(self, name: str, transition_times, transition_info) -> datetime.tzinfo: ...
    @abstractmethod
    def timezone(self, name: str) -> datetime.tzinfo | None: ...
    @abstractmethod
    def uses_pytz(self) -> bool: ...
    @abstractmethod
    def uses_zoneinfo(self) -> bool: ...



from __future__ import annotations

import itertools
import os
import sys
from collections.abc import Iterable

from packaging.requirements import Requirement

from ts_utils.metadata import read_dependencies, read_stubtest_settings
from ts_utils.paths import STUBS_PATH


def get_external_stub_requirements(distributions: Iterable[str] = ()) -> set[Requirement]:
    if not distributions:
        distributions = os.listdir(STUBS_PATH)

    return set(itertools.chain.from_iterable([read_dependencies(distribution).external_pkgs for distribution in distributions]))


def get_stubtest_system_requirements(distributions: Iterable[str] = (), platform: str = sys.platform) -> set[str]:
    if not distributions:
        distributions = os.listdir(STUBS_PATH)

    return set(
        itertools.chain.from_iterable(
            [read_stubtest_settings(distribution).system_requirements_for_platform(platform) for distribution in distributions]
        )
    )



from _typeshed import Incomplete

color01: Incomplete
color02: Incomplete
color03: Incomplete
color04: Incomplete
color05: Incomplete
color06: Incomplete
color07: Incomplete
color08: Incomplete
color09: Incomplete
color10: Incomplete
color01Light: Incomplete
color02Light: Incomplete
color03Light: Incomplete
color04Light: Incomplete
color05Light: Incomplete
color06Light: Incomplete
color07Light: Incomplete
color08Light: Incomplete
color09Light: Incomplete
color10Light: Incomplete
color01Dark: Incomplete
color02Dark: Incomplete
color03Dark: Incomplete
color04Dark: Incomplete
color05Dark: Incomplete
color06Dark: Incomplete
color07Dark: Incomplete
color08Dark: Incomplete
color09Dark: Incomplete
color10Dark: Incomplete
backgroundGrey: Incomplete



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2022 The OpenXLA Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================
"""

import builtins
import typing

import google.protobuf.descriptor
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class TestCompilationEnvironment1(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SOME_FLAG_FIELD_NUMBER: builtins.int
    some_flag: builtins.int
    def __init__(self, *, some_flag: builtins.int | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["some_flag", b"some_flag"]) -> None: ...

global___TestCompilationEnvironment1 = TestCompilationEnvironment1

@typing.final
class TestCompilationEnvironment2(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SOME_OTHER_FLAG_FIELD_NUMBER: builtins.int
    some_other_flag: builtins.int
    def __init__(self, *, some_other_flag: builtins.int | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["some_other_flag", b"some_other_flag"]) -> None: ...

global___TestCompilationEnvironment2 = TestCompilationEnvironment2

@typing.final
class TestCompilationEnvironment3(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    A_THIRD_FLAG_FIELD_NUMBER: builtins.int
    a_third_flag: builtins.int
    def __init__(self, *, a_third_flag: builtins.int | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["a_third_flag", b"a_third_flag"]) -> None: ...

global___TestCompilationEnvironment3 = TestCompilationEnvironment3



from _typeshed import StrPath
from collections.abc import MutableSequence
from subprocess import _ENV

def spawn(
    cmd: MutableSequence[bytes | StrPath],
    search_path: bool = True,
    verbose: bool = False,
    dry_run: bool = False,
    env: _ENV | None = None,
) -> None: ...
def find_executable(executable: str, path: str | None = None) -> str | None: ...



from typing import Any

class ADPersistentSearch:
    connection: Any
    message_id: Any
    base: Any
    scope: Any
    attributes: Any
    controls: Any
    filter: str
    def __init__(self, connection, search_base, search_scope, attributes, streaming, callback) -> None: ...
    def start(self) -> None: ...
    def stop(self, unbind: bool = True) -> None: ...
    def next(self, block: bool = False, timeout=None): ...
    def funnel(self, block: bool = False, timeout=None) -> None: ...



import datetime

from ..core import WesternCalendar

class Latvia(WesternCalendar):
    def get_independence_days(self, year: int) -> list[tuple[datetime.date, str]]: ...
    def get_republic_days(self, year: int) -> list[tuple[datetime.date, str]]: ...



from _typeshed import Incomplete

from authlib.oauth2 import ClientAuth, OAuth2Error, TokenAuth

DEFAULT_HEADERS: Incomplete

class OAuth2Client:
    client_auth_class = ClientAuth
    token_auth_class = TokenAuth
    oauth_error_class = OAuth2Error
    EXTRA_AUTHORIZE_PARAMS: Incomplete
    SESSION_REQUEST_PARAMS: Incomplete
    session: Incomplete
    client_id: Incomplete
    client_secret: Incomplete
    state: Incomplete
    token_endpoint_auth_method: Incomplete
    revocation_endpoint_auth_method: Incomplete
    scope: Incomplete
    redirect_uri: Incomplete
    code_challenge_method: Incomplete
    token_auth: Incomplete
    update_token: Incomplete
    metadata: Incomplete
    compliance_hook: Incomplete
    leeway: Incomplete
    def __init__(
        self,
        session,
        client_id=None,
        client_secret=None,
        token_endpoint_auth_method=None,
        revocation_endpoint_auth_method=None,
        scope=None,
        state=None,
        redirect_uri=None,
        code_challenge_method=None,
        token=None,
        token_placement: str = "header",
        update_token=None,
        leeway: int = 60,
        **metadata,
    ) -> None: ...
    def register_client_auth_method(self, auth) -> None: ...
    def client_auth(self, auth_method): ...
    @property
    def token(self): ...
    @token.setter
    def token(self, token) -> None: ...
    def create_authorization_url(self, url, state=None, code_verifier=None, **kwargs): ...
    def fetch_token(
        self, url=None, body: str = "", method: str = "POST", headers=None, auth=None, grant_type=None, state=None, **kwargs
    ): ...
    def token_from_fragment(self, authorization_response, state=None): ...
    def refresh_token(self, url=None, refresh_token=None, body: str = "", auth=None, headers=None, **kwargs): ...
    def ensure_active_token(self, token=None): ...
    def revoke_token(self, url, token=None, token_type_hint=None, body=None, auth=None, headers=None, **kwargs): ...
    def introspect_token(self, url, token=None, token_type_hint=None, body=None, auth=None, headers=None, **kwargs): ...
    def register_compliance_hook(self, hook_type, hook) -> None: ...
    def parse_response_token(self, resp): ...
    def __del__(self) -> None: ...



import sys
from socket import socket

__all__ = ["stop"]

if sys.platform == "win32":
    __all__ += ["DupSocket"]

    class DupSocket:
        def __init__(self, sock: socket) -> None: ...
        def detach(self) -> socket: ...

else:
    __all__ += ["DupFd"]

    class DupFd:
        def __init__(self, fd: int) -> None: ...
        def detach(self) -> int: ...

def stop(timeout: float | None = None) -> None: ...



from dataclasses import dataclass
from typing import ClassVar

from pandas import DataFrame
from seaborn._core.groupby import GroupBy
from seaborn._core.scales import Scale

@dataclass
class Stat:
    group_by_orient: ClassVar[bool]
    def __call__(self, data: DataFrame, groupby: GroupBy, orient: str, scales: dict[str, Scale]) -> DataFrame: ...



#!/usr/bin/env python3
"""Run mypy on the "tests" and "scripts" directories."""

from __future__ import annotations

import argparse
import subprocess
import sys
from itertools import product
from typing_extensions import TypeAlias

from ts_utils.utils import colored, print_error

ReturnCode: TypeAlias = int

SUPPORTED_PLATFORMS = ("linux", "darwin", "win32")
SUPPORTED_VERSIONS = ("3.14", "3.13", "3.12", "3.11", "3.10", "3.9")
LOWEST_SUPPORTED_VERSION = min(SUPPORTED_VERSIONS, key=lambda x: int(x.split(".")[1]))
DIRECTORIES_TO_TEST = ("scripts", "tests")
EMPTY: list[str] = []

parser = argparse.ArgumentParser(description="Run mypy on typeshed's own code in the `scripts` and `tests` directories.")
parser.add_argument(
    "dir",
    choices=(*DIRECTORIES_TO_TEST, EMPTY),
    nargs="*",
    action="extend",
    help=f"Test only these top-level typeshed directories (defaults to {DIRECTORIES_TO_TEST!r})",
)
parser.add_argument(
    "--platform",
    choices=SUPPORTED_PLATFORMS,
    nargs="*",
    action="extend",
    help="Run mypy for certain OS platforms (defaults to sys.platform)",
)
parser.add_argument(
    "-p",
    "--python-version",
    choices=SUPPORTED_VERSIONS,
    nargs="*",
    action="extend",
    help=f"Run mypy for certain Python versions (defaults to {LOWEST_SUPPORTED_VERSION!r})",
)


def run_mypy_as_subprocess(directory: str, platform: str, version: str) -> ReturnCode:
    command = [
        sys.executable,
        "-m",
        "mypy",
        directory,
        "--platform",
        platform,
        "--python-version",
        version,
        "--strict",
        "--strict-bytes",
        "--local-partial-types",
        "--pretty",
        "--show-traceback",
        "--no-error-summary",
        "--enable-error-code",
        "ignore-without-code",
        # https://github.com/python/mypy/issues/14309
        # "--enable-error-code",
        # "possibly-undefined",
        "--enable-error-code",
        "redundant-expr",
        "--enable-error-code",
        "redundant-self",
        "--custom-typeshed-dir",
        ".",
    ]
    result = subprocess.run(command, capture_output=True, text=True, check=False)
    if result.stderr:
        print_error(result.stderr)
    if result.stdout:
        print_error(result.stdout)
    return result.returncode


def main() -> ReturnCode:
    args = parser.parse_args()
    directories = args.dir or DIRECTORIES_TO_TEST
    platforms = args.platform or [sys.platform]
    versions = args.python_version or [LOWEST_SUPPORTED_VERSION]

    code = 0

    for directory, platform, version in product(directories, platforms, versions):
        print(f'Running "mypy --platform {platform} --python-version {version}" on the "{directory}" directory...')
        code = max(code, run_mypy_as_subprocess(directory, platform, version))

    if code:
        print_error("Test completed with errors")
    else:
        print(colored("Test completed successfully!", "green"))
    return code


if __name__ == "__main__":
    try:
        code = main()
    except KeyboardInterrupt:
        print_error("\n\n!!!\nTest aborted due to KeyboardInterrupt\n!!!")
        code = 1
    raise SystemExit(code)



from networkx.classes.digraph import DiGraph
from networkx.classes.graph import _Node
from networkx.utils.backends import _dispatchable

__all__ = ["is_semiconnected"]

@_dispatchable
def is_semiconnected(G: DiGraph[_Node]) -> bool: ...



from .base import AuthenticationBase

class Passwordless(AuthenticationBase):
    def email(self, email: str, send: str = "link", auth_params: dict[str, str] | None = None): ...
    def sms(self, phone_number: str): ...



from collections.abc import Iterator
from typing import Any, ClassVar

from jmespath.lexer import _LexerTokenizeResult
from jmespath.visitor import Options, _TreeNode

class Parser:
    BINDING_POWER: ClassVar[dict[str, int]]
    tokenizer: Iterator[_LexerTokenizeResult] | None
    def __init__(self, lookahead: int = 2) -> None: ...
    def parse(self, expression: str) -> ParsedResult: ...
    @classmethod
    def purge(cls) -> None: ...

class ParsedResult:
    expression: str
    parsed: _TreeNode
    def __init__(self, expression: str, parsed: _TreeNode) -> None: ...
    def search(self, value: Any, options: Options | None = None) -> Any: ...



from typing import Any, Literal, SupportsIndex, overload
from typing_extensions import TypeAlias

import numpy as np
from numpy.typing import NDArray

from ._enum import ParamEnum
from ._typing import ArrayLike, GeoArray, GeoArrayLikeSeq, OptGeoArrayLike
from .lib import Geometry

__all__ = ["STRtree"]

_BinaryPredicate: TypeAlias = Literal[
    "intersects", "within", "contains", "overlaps", "crosses", "touches", "covers", "covered_by", "contains_properly"
]

class BinaryPredicate(ParamEnum):
    intersects = 1
    within = 2
    contains = 3
    overlaps = 4
    crosses = 5
    touches = 6
    covers = 7
    covered_by = 8
    contains_properly = 9

class STRtree:
    def __init__(self, geoms: GeoArrayLikeSeq, node_capacity: SupportsIndex = 10) -> None: ...
    def __len__(self) -> int: ...
    @property
    def geometries(self) -> GeoArray: ...
    @overload
    def query(
        self, geometry: OptGeoArrayLike, predicate: Literal["dwithin"], distance: ArrayLike[float]
    ) -> NDArray[np.int64]: ...
    @overload
    def query(
        self, geometry: OptGeoArrayLike, predicate: _BinaryPredicate | None = None, distance: object = None
    ) -> NDArray[np.int64]: ...
    # nearest may return `None` if the tree is empty, use the "Any trick"
    @overload
    def nearest(self, geometry: Geometry) -> np.int64 | Any: ...
    @overload
    def nearest(self, geometry: GeoArrayLikeSeq) -> NDArray[np.int64] | Any: ...
    @overload  # return_distance=False
    def query_nearest(
        self,
        geometry: OptGeoArrayLike,
        max_distance: float | None = None,
        return_distance: Literal[False] = False,
        exclusive: bool = False,
        all_matches: bool = True,
    ) -> NDArray[np.int64]: ...
    @overload  # return_distance=True keyword
    def query_nearest(
        self,
        geometry: OptGeoArrayLike,
        max_distance: float | None = None,
        *,
        return_distance: Literal[True],
        exclusive: bool = False,
        all_matches: bool = True,
    ) -> tuple[NDArray[np.int64], NDArray[np.float64]]: ...
    @overload  # return_distance=True positional
    def query_nearest(
        self,
        geometry: OptGeoArrayLike,
        max_distance: float | None,
        return_distance: Literal[True],
        exclusive: bool = False,
        all_matches: bool = True,
    ) -> tuple[NDArray[np.int64], NDArray[np.float64]]: ...
    @overload  # return_distance=bool fallback
    def query_nearest(
        self,
        geometry: OptGeoArrayLike,
        max_distance: float | None = None,
        return_distance: bool = False,
        exclusive: bool = False,
        all_matches: bool = True,
    ) -> NDArray[np.int64] | tuple[NDArray[np.int64], NDArray[np.float64]]: ...



from _typeshed import Incomplete

class Auth0Error(Exception):
    status_code: int
    error_code: str
    message: str
    content: Incomplete | None
    def __init__(self, status_code: int, error_code: str, message: str, content=None) -> None: ...

class RateLimitError(Auth0Error):
    reset_at: int
    def __init__(self, error_code: str, message: str, reset_at: int) -> None: ...

class TokenValidationError(Exception): ...



############################################################################
# Copyright(c) Open Law Library. All rights reserved.                      #
# See ThirdPartyNotices.txt in the project root for additional notices.    #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License")           #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#     http: // www.apache.org/licenses/LICENSE-2.0                         #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
from __future__ import annotations
import asyncio
import json
import logging
import sys
from concurrent.futures import Future
from functools import lru_cache
from itertools import zip_longest
from typing import (
    Callable,
    List,
    Optional,
    Type,
    TypeVar,
    Union,
)


from pygls.capabilities import ServerCapabilitiesBuilder
from pygls.lsp import ConfigCallbackType, ShowDocumentCallbackType
from lsprotocol.types import (
    CLIENT_REGISTER_CAPABILITY,
    CLIENT_UNREGISTER_CAPABILITY,
    EXIT,
    INITIALIZE,
    INITIALIZED,
    METHOD_TO_TYPES,
    NOTEBOOK_DOCUMENT_DID_CHANGE,
    NOTEBOOK_DOCUMENT_DID_CLOSE,
    NOTEBOOK_DOCUMENT_DID_OPEN,
    LOG_TRACE,
    SET_TRACE,
    SHUTDOWN,
    TEXT_DOCUMENT_DID_CHANGE,
    TEXT_DOCUMENT_DID_CLOSE,
    TEXT_DOCUMENT_DID_OPEN,
    TEXT_DOCUMENT_PUBLISH_DIAGNOSTICS,
    WINDOW_LOG_MESSAGE,
    WINDOW_SHOW_DOCUMENT,
    WINDOW_SHOW_MESSAGE,
    WINDOW_WORK_DONE_PROGRESS_CANCEL,
    WORKSPACE_APPLY_EDIT,
    WORKSPACE_CONFIGURATION,
    WORKSPACE_DID_CHANGE_WORKSPACE_FOLDERS,
    WORKSPACE_EXECUTE_COMMAND,
    WORKSPACE_SEMANTIC_TOKENS_REFRESH,
)
from lsprotocol.types import (
    ApplyWorkspaceEditParams,
    Diagnostic,
    DidChangeNotebookDocumentParams,
    DidChangeTextDocumentParams,
    DidChangeWorkspaceFoldersParams,
    DidCloseNotebookDocumentParams,
    DidCloseTextDocumentParams,
    DidOpenNotebookDocumentParams,
    DidOpenTextDocumentParams,
    ExecuteCommandParams,
    InitializeParams,
    InitializeResult,
    LogMessageParams,
    LogTraceParams,
    MessageType,
    PublishDiagnosticsParams,
    RegistrationParams,
    SetTraceParams,
    ShowDocumentParams,
    ShowMessageParams,
    TraceValues,
    UnregistrationParams,
    WorkspaceApplyEditResponse,
    WorkspaceEdit,
    InitializeResultServerInfoType,
    WorkspaceConfigurationParams,
    WorkDoneProgressCancelParams,
)
from pygls.protocol.json_rpc import JsonRPCProtocol
from pygls.protocol.lsp_meta import LSPMeta
from pygls.uris import from_fs_path
from pygls.workspace import Workspace


F = TypeVar("F", bound=Callable)

logger = logging.getLogger(__name__)


def lsp_method(method_name: str) -> Callable[[F], F]:
    def decorator(f: F) -> F:
        f.method_name = method_name  # type: ignore[attr-defined]
        return f

    return decorator


class LanguageServerProtocol(JsonRPCProtocol, metaclass=LSPMeta):
    """A class that represents language server protocol.

    It contains implementations for generic LSP features.

    Attributes:
        workspace(Workspace): In memory workspace
    """

    def __init__(self, server, converter):
        super().__init__(server, converter)

        self._workspace: Optional[Workspace] = None
        self.trace = None

        from pygls.progress import Progress

        self.progress = Progress(self)

        self.server_info = InitializeResultServerInfoType(
            name=server.name,
            version=server.version,
        )

        self._register_builtin_features()

    def _register_builtin_features(self):
        """Registers generic LSP features from this class."""
        for name in dir(self):
            if name in {"workspace"}:
                continue

            attr = getattr(self, name)
            if callable(attr) and hasattr(attr, "method_name"):
                self.fm.add_builtin_feature(attr.method_name, attr)

    @property
    def workspace(self) -> Workspace:
        if self._workspace is None:
            raise RuntimeError(
                "The workspace is not available - has the server been initialized?"
            )

        return self._workspace

    @lru_cache()
    def get_message_type(self, method: str) -> Optional[Type]:
        """Return LSP type definitions, as provided by `lsprotocol`"""
        return METHOD_TO_TYPES.get(method, (None,))[0]

    @lru_cache()
    def get_result_type(self, method: str) -> Optional[Type]:
        return METHOD_TO_TYPES.get(method, (None, None))[1]

    def apply_edit(
        self, edit: WorkspaceEdit, label: Optional[str] = None
    ) -> WorkspaceApplyEditResponse:
        """Sends apply edit request to the client."""
        return self.send_request(
            WORKSPACE_APPLY_EDIT, ApplyWorkspaceEditParams(edit=edit, label=label)
        )

    def apply_edit_async(
        self, edit: WorkspaceEdit, label: Optional[str] = None
    ) -> WorkspaceApplyEditResponse:
        """Sends apply edit request to the client. Should be called with `await`"""
        return self.send_request_async(
            WORKSPACE_APPLY_EDIT, ApplyWorkspaceEditParams(edit=edit, label=label)
        )

    @lsp_method(EXIT)
    def lsp_exit(self, *args) -> None:
        """Stops the server process."""
        if self.transport is not None:
            self.transport.close()

        sys.exit(0 if self._shutdown else 1)

    @lsp_method(INITIALIZE)
    def lsp_initialize(self, params: InitializeParams) -> InitializeResult:
        """Method that initializes language server.
        It will compute and return server capabilities based on
        registered features.
        """
        logger.info("Language server initialized %s", params)

        self._server.process_id = params.process_id

        text_document_sync_kind = self._server._text_document_sync_kind
        notebook_document_sync = self._server._notebook_document_sync

        # Initialize server capabilities
        self.client_capabilities = params.capabilities
        self.server_capabilities = ServerCapabilitiesBuilder(
            self.client_capabilities,
            set({**self.fm.features, **self.fm.builtin_features}.keys()),
            self.fm.feature_options,
            list(self.fm.commands.keys()),
            text_document_sync_kind,
            notebook_document_sync,
        ).build()
        logger.debug(
            "Server capabilities: %s",
            json.dumps(self.server_capabilities, default=self._serialize_message),
        )

        root_path = params.root_path
        root_uri = params.root_uri
        if root_path is not None and root_uri is None:
            root_uri = from_fs_path(root_path)

        # Initialize the workspace
        workspace_folders = params.workspace_folders or []
        self._workspace = Workspace(
            root_uri,
            text_document_sync_kind,
            workspace_folders,
            self.server_capabilities.position_encoding,
        )

        self.trace = TraceValues.Off

        return InitializeResult(
            capabilities=self.server_capabilities,
            server_info=self.server_info,
        )

    @lsp_method(INITIALIZED)
    def lsp_initialized(self, *args) -> None:
        """Notification received when client and server are connected."""
        pass

    @lsp_method(SHUTDOWN)
    def lsp_shutdown(self, *args) -> None:
        """Request from client which asks server to shutdown."""
        for future in self._request_futures.values():
            future.cancel()

        self._shutdown = True
        return None

    @lsp_method(TEXT_DOCUMENT_DID_CHANGE)
    def lsp_text_document__did_change(
        self, params: DidChangeTextDocumentParams
    ) -> None:
        """Updates document's content.
        (Incremental(from server capabilities); not configurable for now)
        """
        for change in params.content_changes:
            self.workspace.update_text_document(params.text_document, change)

    @lsp_method(TEXT_DOCUMENT_DID_CLOSE)
    def lsp_text_document__did_close(self, params: DidCloseTextDocumentParams) -> None:
        """Removes document from workspace."""
        self.workspace.remove_text_document(params.text_document.uri)

    @lsp_method(TEXT_DOCUMENT_DID_OPEN)
    def lsp_text_document__did_open(self, params: DidOpenTextDocumentParams) -> None:
        """Puts document to the workspace."""
        self.workspace.put_text_document(params.text_document)

    @lsp_method(NOTEBOOK_DOCUMENT_DID_OPEN)
    def lsp_notebook_document__did_open(
        self, params: DidOpenNotebookDocumentParams
    ) -> None:
        """Put a notebook document into the workspace"""
        self.workspace.put_notebook_document(params)

    @lsp_method(NOTEBOOK_DOCUMENT_DID_CHANGE)
    def lsp_notebook_document__did_change(
        self, params: DidChangeNotebookDocumentParams
    ) -> None:
        """Update a notebook's contents"""
        self.workspace.update_notebook_document(params)

    @lsp_method(NOTEBOOK_DOCUMENT_DID_CLOSE)
    def lsp_notebook_document__did_close(
        self, params: DidCloseNotebookDocumentParams
    ) -> None:
        """Remove a notebook document from the workspace."""
        self.workspace.remove_notebook_document(params)

    @lsp_method(SET_TRACE)
    def lsp_set_trace(self, params: SetTraceParams) -> None:
        """Changes server trace value."""
        self.trace = params.value

    @lsp_method(WORKSPACE_DID_CHANGE_WORKSPACE_FOLDERS)
    def lsp_workspace__did_change_workspace_folders(
        self, params: DidChangeWorkspaceFoldersParams
    ) -> None:
        """Adds/Removes folders from the workspace."""
        logger.info("Workspace folders changed: %s", params)

        added_folders = params.event.added or []
        removed_folders = params.event.removed or []

        for f_add, f_remove in zip_longest(added_folders, removed_folders):
            if f_add:
                self.workspace.add_folder(f_add)
            if f_remove:
                self.workspace.remove_folder(f_remove.uri)

    @lsp_method(WORKSPACE_EXECUTE_COMMAND)
    def lsp_workspace__execute_command(
        self, params: ExecuteCommandParams, msg_id: str
    ) -> None:
        """Executes commands with passed arguments and returns a value."""
        cmd_handler = self.fm.commands[params.command]
        self._execute_request(msg_id, cmd_handler, params.arguments)

    @lsp_method(WINDOW_WORK_DONE_PROGRESS_CANCEL)
    def lsp_work_done_progress_cancel(
        self, params: WorkDoneProgressCancelParams
    ) -> None:
        """Received a progress cancellation from client."""
        future = self.progress.tokens.get(params.token)
        if future is None:
            logger.warning(
                "Ignoring work done progress cancel for unknown token %s", params.token
            )
        else:
            future.cancel()

    def get_configuration(
        self,
        params: WorkspaceConfigurationParams,
        callback: Optional[ConfigCallbackType] = None,
    ) -> Future:
        """Sends configuration request to the client.

        Args:
            params(WorkspaceConfigurationParams): WorkspaceConfigurationParams from lsp specs
            callback(callable): Callabe which will be called after
                                response from the client is received
        Returns:
            concurrent.futures.Future object that will be resolved once a
            response has been received
        """
        return self.send_request(WORKSPACE_CONFIGURATION, params, callback)

    def get_configuration_async(
        self, params: WorkspaceConfigurationParams
    ) -> asyncio.Future:
        """Calls `get_configuration` method but designed to use with coroutines

        Args:
            params(WorkspaceConfigurationParams): WorkspaceConfigurationParams from lsp specs
        Returns:
            asyncio.Future that can be awaited
        """
        return asyncio.wrap_future(self.get_configuration(params))

    def log_trace(self, message: str, verbose: Optional[str] = None) -> None:
        """Sends trace notification to the client."""
        if self.trace == TraceValues.Off:
            return

        params = LogTraceParams(message=message)
        if verbose and self.trace == TraceValues.Verbose:
            params.verbose = verbose

        self.notify(LOG_TRACE, params)

    def _publish_diagnostics_deprecator(
        self,
        params_or_uri: Union[str, PublishDiagnosticsParams],
        diagnostics: Optional[List[Diagnostic]],
        version: Optional[int],
        **kwargs,
    ) -> PublishDiagnosticsParams:
        if isinstance(params_or_uri, str):
            message = "DEPRECATION: "
            "`publish_diagnostics("
            "self, doc_uri: str, diagnostics: List[Diagnostic], version: Optional[int] = None)`"
            "will be replaced with `publish_diagnostics(self, params: PublishDiagnosticsParams)`"
            logging.warning(message)

            params = self._construct_publish_diagnostic_type(
                params_or_uri, diagnostics, version, **kwargs
            )
        else:
            params = params_or_uri
        return params

    def _construct_publish_diagnostic_type(
        self,
        uri: str,
        diagnostics: Optional[List[Diagnostic]],
        version: Optional[int],
        **kwargs,
    ) -> PublishDiagnosticsParams:
        if diagnostics is None:
            diagnostics = []

        args = {
            **{"uri": uri, "diagnostics": diagnostics, "version": version},
            **kwargs,
        }

        params = PublishDiagnosticsParams(**args)  # type:ignore
        return params

    def publish_diagnostics(
        self,
        params_or_uri: Union[str, PublishDiagnosticsParams],
        diagnostics: Optional[List[Diagnostic]] = None,
        version: Optional[int] = None,
        **kwargs,
    ):
        """Sends diagnostic notification to the client.

        .. deprecated:: 1.0.1

           Passing ``(uri, diagnostics, version)`` as arguments is deprecated.
           Pass an instance of :class:`~lsprotocol.types.PublishDiagnosticParams`
           instead.

        Parameters
        ----------
        params_or_uri
           The :class:`~lsprotocol.types.PublishDiagnosticParams` to send to the client.

        diagnostics
           *Deprecated*. The diagnostics to publish

        version
           *Deprecated*: The version number
        """
        params = self._publish_diagnostics_deprecator(
            params_or_uri, diagnostics, version, **kwargs
        )
        self.notify(TEXT_DOCUMENT_PUBLISH_DIAGNOSTICS, params)

    def register_capability(
        self, params: RegistrationParams, callback: Optional[Callable[[], None]] = None
    ) -> Future:
        """Register a new capability on the client.

        Args:
            params(RegistrationParams): RegistrationParams from lsp specs
            callback(callable): Callabe which will be called after
                                response from the client is received
        Returns:
            concurrent.futures.Future object that will be resolved once a
            response has been received
        """
        return self.send_request(CLIENT_REGISTER_CAPABILITY, params, callback)

    def register_capability_async(self, params: RegistrationParams) -> asyncio.Future:
        """Register a new capability on the client.

        Args:
            params(RegistrationParams): RegistrationParams from lsp specs

        Returns:
            asyncio.Future object that will be resolved once a
            response has been received
        """
        return asyncio.wrap_future(self.register_capability(params, None))

    def semantic_tokens_refresh(
        self, callback: Optional[Callable[[], None]] = None
    ) -> Future:
        """Requesting a refresh of all semantic tokens.

        Args:
            callback(callable): Callabe which will be called after
                                response from the client is received

        Returns:
            concurrent.futures.Future object that will be resolved once a
            response has been received
        """
        return self.send_request(WORKSPACE_SEMANTIC_TOKENS_REFRESH, callback=callback)

    def semantic_tokens_refresh_async(self) -> asyncio.Future:
        """Requesting a refresh of all semantic tokens.

        Returns:
            asyncio.Future object that will be resolved once a
            response has been received
        """
        return asyncio.wrap_future(self.semantic_tokens_refresh(None))

    def show_document(
        self,
        params: ShowDocumentParams,
        callback: Optional[ShowDocumentCallbackType] = None,
    ) -> Future:
        """Display a particular document in the user interface.

        Args:
            params(ShowDocumentParams): ShowDocumentParams from lsp specs
            callback(callable): Callabe which will be called after
                                response from the client is received

        Returns:
            concurrent.futures.Future object that will be resolved once a
            response has been received
        """
        return self.send_request(WINDOW_SHOW_DOCUMENT, params, callback)

    def show_document_async(self, params: ShowDocumentParams) -> asyncio.Future:
        """Display a particular document in the user interface.

        Args:
            params(ShowDocumentParams): ShowDocumentParams from lsp specs

        Returns:
            asyncio.Future object that will be resolved once a
            response has been received
        """
        return asyncio.wrap_future(self.show_document(params, None))

    def show_message(self, message, msg_type=MessageType.Info):
        """Sends message to the client to display message."""
        self.notify(
            WINDOW_SHOW_MESSAGE, ShowMessageParams(type=msg_type, message=message)
        )

    def show_message_log(self, message, msg_type=MessageType.Log):
        """Sends message to the client's output channel."""
        self.notify(
            WINDOW_LOG_MESSAGE, LogMessageParams(type=msg_type, message=message)
        )

    def unregister_capability(
        self,
        params: UnregistrationParams,
        callback: Optional[Callable[[], None]] = None,
    ) -> Future:
        """Unregister a new capability on the client.

        Args:
            params(UnregistrationParams): UnregistrationParams from lsp specs
            callback(callable): Callabe which will be called after
                                response from the client is received
        Returns:
            concurrent.futures.Future object that will be resolved once a
            response has been received
        """
        return self.send_request(CLIENT_UNREGISTER_CAPABILITY, params, callback)

    def unregister_capability_async(
        self, params: UnregistrationParams
    ) -> asyncio.Future:
        """Unregister a new capability on the client.

        Args:
            params(UnregistrationParams): UnregistrationParams from lsp specs
            callback(callable): Callabe which will be called after
                                response from the client is received
        Returns:
            asyncio.Future object that will be resolved once a
            response has been received
        """
        return asyncio.wrap_future(self.unregister_capability(params, None))



from braintree.graphql.inputs.create_customer_session_input import CreateCustomerSessionInput as CreateCustomerSessionInput
from braintree.graphql.inputs.customer_recommendations_input import CustomerRecommendationsInput as CustomerRecommendationsInput
from braintree.graphql.inputs.customer_session_input import CustomerSessionInput as CustomerSessionInput
from braintree.graphql.inputs.monetary_amount_input import MonetaryAmountInput as MonetaryAmountInput
from braintree.graphql.inputs.paypal_payee_input import PayPalPayeeInput as PayPalPayeeInput
from braintree.graphql.inputs.paypal_purchase_unit_input import PayPalPurchaseUnitInput as PayPalPurchaseUnitInput
from braintree.graphql.inputs.phone_input import PhoneInput as PhoneInput
from braintree.graphql.inputs.update_customer_session_input import UpdateCustomerSessionInput as UpdateCustomerSessionInput



from typing import ClassVar, Final, Protocol, type_check_only
from typing_extensions import Self

from docutils.utils import Reporter

__docformat__: Final = "reStructuredText"

@type_check_only
class _LanguageModule(Protocol):
    labels: dict[str, str]
    author_separators: list[str]
    bibliographic_fields: list[str]

class LanguageImporter:
    packages: ClassVar[tuple[str, ...]]
    warn_msg: ClassVar[str]
    fallback: ClassVar[str]
    cache: dict[str, _LanguageModule]
    def __init__(self) -> None: ...
    def import_from_packages(self, name: str, reporter: Reporter | None = None) -> _LanguageModule: ...
    def check_content(self, module: _LanguageModule) -> None: ...
    def __call__(self, language_code: str, reporter: Reporter | None = None) -> _LanguageModule: ...
    def __class_getitem__(cls, name) -> type[Self]: ...

get_language: LanguageImporter



from _typeshed import Incomplete

from ..rest import RestClientOptions
from ..types import TimeoutType

class Emails:
    domain: Incomplete
    protocol: Incomplete
    client: Incomplete
    def __init__(
        self,
        domain: str,
        token: str,
        telemetry: bool = True,
        timeout: TimeoutType = 5.0,
        protocol: str = "https",
        rest_options: RestClientOptions | None = None,
    ) -> None: ...
    def get(self, fields: list[str] | None = None, include_fields: bool = True) -> dict[str, Incomplete]: ...
    async def get_async(self, fields: list[str] | None = None, include_fields: bool = True) -> dict[str, Incomplete]: ...
    def config(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def config_async(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def delete(self): ...
    async def delete_async(self): ...
    def update(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def update_async(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...



from _typeshed import Incomplete
from collections.abc import Generator

__all__ = ["ISMAGS"]

class ISMAGS:
    graph: Incomplete
    subgraph: Incomplete
    node_equality: Incomplete
    edge_equality: Incomplete

    def __init__(self, graph, subgraph, node_match=None, edge_match=None, cache=None) -> None: ...
    def find_isomorphisms(self, symmetry: bool = True) -> Generator[Incomplete, Incomplete, Incomplete]: ...
    def largest_common_subgraph(self, symmetry: bool = True) -> Generator[Incomplete, Incomplete, None]: ...
    def analyze_symmetry(self, graph, node_partitions, edge_colors): ...
    def is_isomorphic(self, symmetry: bool = False) -> bool: ...
    def subgraph_is_isomorphic(self, symmetry: bool = False) -> bool: ...
    def isomorphisms_iter(self, symmetry: bool = True) -> Generator[Incomplete, Incomplete, None]: ...
    def subgraph_isomorphisms_iter(self, symmetry: bool = True): ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class GraphDebugInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class FileLineCol(google.protobuf.message.Message):
        """This represents a file/line location in the source code."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        FILE_INDEX_FIELD_NUMBER: builtins.int
        LINE_FIELD_NUMBER: builtins.int
        COL_FIELD_NUMBER: builtins.int
        FUNC_FIELD_NUMBER: builtins.int
        CODE_FIELD_NUMBER: builtins.int
        file_index: builtins.int
        """File name index, which can be used to retrieve the file name string from
        `files`. The value should be between 0 and (len(files)-1)
        """
        line: builtins.int
        """Line number in the file."""
        col: builtins.int
        """Col number in the file line."""
        func: builtins.str
        """Name of function contains the file line."""
        code: builtins.str
        """Source code contained in this file line."""
        def __init__(
            self,
            *,
            file_index: builtins.int | None = ...,
            line: builtins.int | None = ...,
            col: builtins.int | None = ...,
            func: builtins.str | None = ...,
            code: builtins.str | None = ...,
        ) -> None: ...
        def HasField(
            self,
            field_name: typing.Literal[
                "code", b"code", "col", b"col", "file_index", b"file_index", "func", b"func", "line", b"line"
            ],
        ) -> builtins.bool: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "code", b"code", "col", b"col", "file_index", b"file_index", "func", b"func", "line", b"line"
            ],
        ) -> None: ...

    @typing.final
    class StackTrace(google.protobuf.message.Message):
        """This represents a stack trace which is a ordered list of `FileLineCol`."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        FILE_LINE_COLS_FIELD_NUMBER: builtins.int
        FRAME_ID_FIELD_NUMBER: builtins.int
        @property
        def file_line_cols(
            self,
        ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___GraphDebugInfo.FileLineCol]:
            """Deprecated."""

        @property
        def frame_id(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
        def __init__(
            self,
            *,
            file_line_cols: collections.abc.Iterable[global___GraphDebugInfo.FileLineCol] | None = ...,
            frame_id: collections.abc.Iterable[builtins.int] | None = ...,
        ) -> None: ...
        def ClearField(
            self, field_name: typing.Literal["file_line_cols", b"file_line_cols", "frame_id", b"frame_id"]
        ) -> None: ...

    @typing.final
    class FramesByIdEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        @property
        def value(self) -> global___GraphDebugInfo.FileLineCol: ...
        def __init__(
            self, *, key: builtins.int | None = ..., value: global___GraphDebugInfo.FileLineCol | None = ...
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    @typing.final
    class TracesByIdEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        @property
        def value(self) -> global___GraphDebugInfo.StackTrace: ...
        def __init__(self, *, key: builtins.int | None = ..., value: global___GraphDebugInfo.StackTrace | None = ...) -> None: ...
        def HasField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    @typing.final
    class TracesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___GraphDebugInfo.StackTrace: ...
        def __init__(self, *, key: builtins.str | None = ..., value: global___GraphDebugInfo.StackTrace | None = ...) -> None: ...
        def HasField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    @typing.final
    class NameToTraceIdEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.int
        def __init__(self, *, key: builtins.str | None = ..., value: builtins.int | None = ...) -> None: ...
        def HasField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    FILES_FIELD_NUMBER: builtins.int
    FRAMES_BY_ID_FIELD_NUMBER: builtins.int
    TRACES_BY_ID_FIELD_NUMBER: builtins.int
    TRACES_FIELD_NUMBER: builtins.int
    NAME_TO_TRACE_ID_FIELD_NUMBER: builtins.int
    @property
    def files(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """This stores all the source code file names and can be indexed by the
        `file_index`.
        """

    @property
    def frames_by_id(self) -> google.protobuf.internal.containers.MessageMap[builtins.int, global___GraphDebugInfo.FileLineCol]:
        """Stack traces and frames are uniqueified during construction. These maps
        index from the unique id for a frame/trace to the value.
        """

    @property
    def traces_by_id(
        self,
    ) -> google.protobuf.internal.containers.MessageMap[builtins.int, global___GraphDebugInfo.StackTrace]: ...
    @property
    def traces(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___GraphDebugInfo.StackTrace]:
        """Deprecated."""

    @property
    def name_to_trace_id(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.int]:
        """This maps a node name to a trace id contained in `traces_by_id`.

        The map key is a mangling of the containing function and op name with
        syntax:
          op.name '@' func_name
        For ops in the top-level graph, the func_name is the empty string and hence
        the `@` may be ommitted.
        Note that op names are restricted to a small number of characters which
        exclude '@', making it impossible to collide keys of this form. Function
        names accept a much wider set of characters.
        It would be preferable to avoid mangling and use a tuple key of (op.name,
        func_name), but this is not supported with protocol buffers.
        """

    def __init__(
        self,
        *,
        files: collections.abc.Iterable[builtins.str] | None = ...,
        frames_by_id: collections.abc.Mapping[builtins.int, global___GraphDebugInfo.FileLineCol] | None = ...,
        traces_by_id: collections.abc.Mapping[builtins.int, global___GraphDebugInfo.StackTrace] | None = ...,
        traces: collections.abc.Mapping[builtins.str, global___GraphDebugInfo.StackTrace] | None = ...,
        name_to_trace_id: collections.abc.Mapping[builtins.str, builtins.int] | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "files",
            b"files",
            "frames_by_id",
            b"frames_by_id",
            "name_to_trace_id",
            b"name_to_trace_id",
            "traces",
            b"traces",
            "traces_by_id",
            b"traces_by_id",
        ],
    ) -> None: ...

global___GraphDebugInfo = GraphDebugInfo



from _typeshed import Incomplete, SupportsLenAndGetItem
from collections.abc import Generator, Iterable
from logging import Logger
from typing import ClassVar, Protocol, TypeVar, overload, type_check_only
from typing_extensions import TypeAlias

from .enums import AccessPermission, EncryptionMethod
from .fpdf import FPDF
from .syntax import Name, PDFObject

_Key: TypeAlias = SupportsLenAndGetItem[int]
_T_co = TypeVar("_T_co", covariant=True)

LOGGER: Logger

import_error: ImportError | None

@type_check_only
class _SupportsGetItem(Protocol[_T_co]):
    def __getitem__(self, k: int, /) -> _T_co: ...

class ARC4:
    MOD: ClassVar[int]
    def KSA(self, key: _Key) -> list[int]: ...
    def PRGA(self, S: _SupportsGetItem[int]) -> Generator[int, None, None]: ...
    def encrypt(self, key: _Key, text: Iterable[int]) -> list[int]: ...

class CryptFilter:
    type: Name
    c_f_m: Name
    length: int
    def __init__(self, mode: str, length: int) -> None: ...
    def serialize(self) -> str: ...

class EncryptionDictionary(PDFObject):
    filter: Name
    length: int
    r: int
    o: str
    u: str
    v: int
    p: int
    encrypt_metadata: str  # not always defined
    c_f: str  # not always defined
    stm_f: Name
    str_f: Name
    def __init__(self, security_handler: StandardSecurityHandler) -> None: ...

class StandardSecurityHandler:
    DEFAULT_PADDING: ClassVar[bytes]
    fpdf: FPDF
    access_permission: int
    owner_password: str
    user_password: str
    encryption_method: EncryptionMethod | None
    cf: CryptFilter | None
    key_length: int
    version: int
    revision: int
    encrypt_metadata: bool

    # The following fields are only defined after a call to generate_passwords().
    file_id: Incomplete
    info_id: Incomplete
    o: str
    k: str
    u: str
    # The following field is only defined after a call to generate_user_password_rev6().
    ue: Incomplete
    # The following field is only defined after a call to generate_owner_password_rev6().
    oe: Incomplete
    # The following field is only defined after a call to generate_perms_rev6().
    perms_rev6: Incomplete

    def __init__(
        self,
        fpdf: FPDF,
        owner_password: str,
        user_password: str | None = None,
        permission: AccessPermission = ...,
        encryption_method: EncryptionMethod = ...,
        encrypt_metadata: bool = False,
    ) -> None: ...
    def generate_passwords(self, file_id: str) -> None: ...
    def get_encryption_obj(self) -> EncryptionDictionary: ...
    @overload
    def encrypt(self, text: bytes | bytearray, obj_id: int) -> bytes: ...
    @overload
    def encrypt(self, text: str, obj_id: int) -> str: ...
    def encrypt_string(self, string: str, obj_id: int) -> str: ...
    def encrypt_stream(self, stream: bytes, obj_id: int) -> bytes: ...
    def is_aes_algorithm(self) -> bool: ...
    def encrypt_bytes(self, data: bytes, obj_id: int) -> list[int]: ...
    def encrypt_AES_cryptography(self, key: bytes, data: bytes) -> bytes: ...
    @classmethod
    def get_random_bytes(cls, size: int) -> bytes: ...
    @classmethod
    def prepare_string(cls, string: str) -> bytes: ...
    def padded_password(self, password: str) -> bytearray: ...
    def generate_owner_password(self) -> str: ...
    def generate_user_password(self) -> str: ...
    @classmethod
    def compute_hash(cls, input_password: bytes, salt: bytes, user_key: bytes = ...) -> bytes: ...
    def generate_user_password_rev6(self) -> None: ...
    def generate_owner_password_rev6(self) -> None: ...
    def generate_perms_rev6(self) -> None: ...
    def generate_encryption_key(self) -> bytes: ...

def md5(data: bytes | bytearray) -> bytes: ...
def int32(n: int) -> int: ...



from _typeshed import Incomplete

from authlib.jose import JWTClaims

__all__ = ["IDToken", "CodeIDToken", "ImplicitIDToken", "HybridIDToken", "UserInfo", "get_claim_cls_by_response_type"]

class IDToken(JWTClaims):
    ESSENTIAL_CLAIMS: Incomplete
    def validate(self, now=None, leeway: int = 0) -> None: ...
    def validate_auth_time(self) -> None: ...
    def validate_nonce(self) -> None: ...
    def validate_acr(self): ...
    def validate_amr(self) -> None: ...
    def validate_azp(self) -> None: ...
    def validate_at_hash(self) -> None: ...

class CodeIDToken(IDToken):
    RESPONSE_TYPES: Incomplete

class ImplicitIDToken(IDToken):
    RESPONSE_TYPES: Incomplete
    ESSENTIAL_CLAIMS: Incomplete
    def validate_at_hash(self) -> None: ...

class HybridIDToken(ImplicitIDToken):
    RESPONSE_TYPES: Incomplete
    def validate(self, now=None, leeway: int = 0) -> None: ...
    def validate_c_hash(self) -> None: ...

class UserInfo(dict[str, object]):
    REGISTERED_CLAIMS: list[str]
    SCOPES_CLAIMS_MAPPING: dict[str, list[str]]
    def filter(self, scope: str) -> UserInfo: ...
    def __getattr__(self, key): ...

def get_claim_cls_by_response_type(response_type): ...



from _typeshed import Unused
from typing import Any

DEFAULTS: dict[str, Any]  # Configuration values can be strings, booleans, callables, etc.
DEPRECATED_SETTINGS: list[str]

def is_callable(value: Any) -> bool: ...  # Accepts any value to test if it's callable

class Settings:
    # Setting values can be of any type, so getter and setter methods return/accept Any
    def __getattr__(self, name: str) -> Any: ...  # Returns setting values of various types
    def get_setting(self, setting: str) -> Any: ...  # Setting values vary by configuration option
    def change_setting(
        self, setting: str, value: Any, enter: bool, **kwargs: Unused
    ) -> None: ...  # Accepts any setting value type

settings: Settings



from logging import Logger
from typing import ClassVar, Protocol, type_check_only

from .spec import Connection

@type_check_only
class _Credentials(Protocol):
    TYPE: ClassVar[str]
    erase_on_connect: bool
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def response_for(self, start: Connection.Start) -> tuple[str | None, bytes | None]: ...
    def erase_credentials(self) -> None: ...

LOGGER: Logger

class PlainCredentials:
    TYPE: ClassVar[str]
    erase_on_connect: bool
    username: str
    password: str
    def __init__(self, username: str, password: str, erase_on_connect: bool = False) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def response_for(self, start: Connection.Start) -> tuple[str | None, bytes | None]: ...
    def erase_credentials(self) -> None: ...

class ExternalCredentials:
    TYPE: ClassVar[str]
    erase_on_connect: bool
    def __init__(self) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def response_for(self, start: Connection.Start) -> tuple[str | None, bytes | None]: ...
    def erase_credentials(self) -> None: ...

VALID_TYPES: list[_Credentials]



import unittest
from _typeshed import Unused
from typing import Any

__all__ = ["TestCase", "LegacyTestCase", "Kwargs"]

class TestCase(unittest.TestCase):
    default_kwargs: dict[str, Any]  # taken from source code
    def assertMarkdownRenders(
        self,
        source: str,
        expected: str,
        expected_attrs: dict[str, Any] | None = None,  # values passing to self.assertEqual()
        **kwargs,
    ) -> None: ...
    def dedent(self, text: str) -> str: ...

class recursionlimit:
    limit: int
    old_limit: int
    def __init__(self, limit: int) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, type: Unused, value: Unused, tb: Unused) -> None: ...

class Kwargs(dict[str, Any]): ...

class LegacyTestMeta(type):
    def __new__(cls, name: str, bases: tuple[type, ...], dct: dict[str, Any]): ...  # dct is namespace argument for type.__new__()

class LegacyTestCase(unittest.TestCase, metaclass=LegacyTestMeta): ...



from .segment import Segment
from .subsegment import Subsegment

class DummySegment(Segment):
    def __init__(self, name: str = "dummy") -> None: ...

class DummySubsegment(Subsegment):
    def __init__(self, segment, name: str = "dummy") -> None: ...



from _typeshed import FileDescriptorOrPath
from collections.abc import Iterable
from typing import BinaryIO, type_check_only

from ..base import AsyncBase, AsyncIndirectBase

@type_check_only
class _UnknownAsyncTextIO(AsyncBase[str]):
    async def close(self) -> None: ...
    async def flush(self) -> None: ...
    async def isatty(self) -> bool: ...
    async def read(self, size: int | None = ..., /) -> str: ...
    async def readline(self, size: int = ..., /) -> str: ...
    async def readlines(self, hint: int = ..., /) -> list[str]: ...
    async def seek(self, offset: int, whence: int = ..., /) -> int: ...
    async def seekable(self) -> bool: ...
    async def tell(self) -> int: ...
    async def truncate(self, size: int | None = ..., /) -> int: ...
    async def writable(self) -> bool: ...
    async def write(self, b: str, /) -> int: ...
    async def writelines(self, lines: Iterable[str], /) -> None: ...
    def detach(self) -> BinaryIO: ...
    def fileno(self) -> int: ...
    def readable(self) -> bool: ...
    @property
    def buffer(self) -> BinaryIO: ...
    @property
    def closed(self) -> bool: ...
    @property
    def encoding(self) -> str: ...
    @property
    def errors(self) -> str | None: ...
    @property
    def line_buffering(self) -> bool: ...
    @property
    def newlines(self) -> str | tuple[str, ...] | None: ...
    @property
    def name(self) -> FileDescriptorOrPath: ...
    @property
    def mode(self) -> str: ...

class AsyncTextIOWrapper(_UnknownAsyncTextIO): ...
class AsyncTextIndirectIOWrapper(AsyncIndirectBase[str], _UnknownAsyncTextIO): ...



import sys
from collections.abc import Iterator
from re import Pattern
from typing import Final, Generic, TypedDict, TypeVar, overload, type_check_only

from markdown.core import Markdown

_T = TypeVar("_T")

BLOCK_LEVEL_ELEMENTS: Final[list[str]]
STX: Final[str]
ETX: Final[str]
INLINE_PLACEHOLDER_PREFIX: Final[str]
INLINE_PLACEHOLDER: Final[str]
INLINE_PLACEHOLDER_RE: Final[Pattern[str]]
AMP_SUBSTITUTE: Final[str]
HTML_PLACEHOLDER: Final[str]
HTML_PLACEHOLDER_RE: Final[Pattern[str]]
TAG_PLACEHOLDER: Final[str]
RTL_BIDI_RANGES: Final[tuple[tuple[str, str], tuple[str, str]]]

if sys.version_info >= (3, 10):
    from importlib import metadata
    def get_installed_extensions() -> metadata.EntryPoints: ...

else:
    def get_installed_extensions(): ...

def deprecated(message: str, stacklevel: int = 2): ...
@overload
def parseBoolValue(value: str) -> bool: ...
@overload
def parseBoolValue(value: str | None, fail_on_errors: bool = True, preserve_none: bool = False) -> bool | None: ...
def code_escape(text: str) -> str: ...
def nearing_recursion_limit() -> bool: ...

class AtomicString(str): ...

class Processor:
    md: Markdown
    def __init__(self, md: Markdown | None = None) -> None: ...

@type_check_only
class _TagData(TypedDict):
    tag: str
    attrs: dict[str, str]
    left_index: int
    right_index: int

class HtmlStash:
    html_counter: int
    rawHtmlBlocks: list[str]
    tag_counter: int
    tag_data: list[_TagData]
    def __init__(self) -> None: ...
    def store(self, html: str) -> str: ...
    def reset(self) -> None: ...
    def get_placeholder(self, key: int) -> str: ...
    def store_tag(self, tag: str, attrs: dict[str, str], left_index: int, right_index: int) -> str: ...

class Registry(Generic[_T]):
    def __init__(self) -> None: ...
    def __contains__(self, item: str | _T) -> bool: ...
    def __iter__(self) -> Iterator[_T]: ...
    @overload
    def __getitem__(self, key: slice) -> Registry[_T]: ...
    @overload
    def __getitem__(self, key: str | int) -> _T: ...
    def __len__(self) -> int: ...
    def get_index_for_name(self, name: str) -> int: ...
    def register(self, item: _T, name: str, priority: float) -> None: ...
    def deregister(self, name: str, strict: bool = True) -> None: ...



from _typeshed import Incomplete
from collections.abc import Iterable, Mapping
from datetime import date, datetime, timedelta
from typing import Any, Protocol, type_check_only
from typing_extensions import TypeAlias

from matplotlib.colors import Colormap, Normalize
from numpy import ndarray
from pandas import DataFrame, Index, Series, Timedelta, Timestamp

@type_check_only
class SupportsDataFrame(Protocol):
    # `__dataframe__` should return pandas.core.interchange.dataframe_protocol.DataFrame
    # but this class needs to be defined as a Protocol, not as an ABC.
    def __dataframe__(self, nan_as_null: bool = ..., allow_copy: bool = ...): ...

ColumnName: TypeAlias = str | bytes | date | datetime | timedelta | bool | complex | Timestamp | Timedelta
Vector: TypeAlias = Series[Any] | Index[Any] | ndarray[Any, Any]
VariableSpec: TypeAlias = ColumnName | Vector | None
VariableSpecList: TypeAlias = list[VariableSpec] | Index[Any] | None
DataSource: TypeAlias = DataFrame | SupportsDataFrame | Mapping[Any, Incomplete] | None
OrderSpec: TypeAlias = Iterable[str] | None
NormSpec: TypeAlias = tuple[float | None, float | None] | Normalize | None
PaletteSpec: TypeAlias = str | list[Incomplete] | dict[Incomplete, Incomplete] | Colormap | None
DiscreteValueSpec: TypeAlias = dict[Incomplete, Incomplete] | list[Incomplete] | None
ContinuousValueSpec: TypeAlias = tuple[float, float] | list[float] | dict[Any, float] | None

class Default: ...
class Deprecated: ...

default: Default
deprecated: Deprecated



import abc
import enum
import io
from _typeshed import Unused
from collections.abc import Callable, Iterable, Mapping
from email.message import Message
from logging import Logger
from typing import IO, Any, Final
from typing_extensions import Self, TypeAlias

from ..cookies import YoutubeDLCookieJar
from ..utils._utils import _YDLLogger
from ..utils.networking import HTTPHeaderDict

DEFAULT_TIMEOUT: Final = 20
_RequestData: TypeAlias = bytes | Iterable[bytes] | IO[Any] | None
_Preference: TypeAlias = Callable[[RequestHandler, Request], int]

def register_preference(*handlers: type[RequestHandler]) -> Callable[..., object]: ...

class RequestDirector:
    handlers: dict[str, RequestHandler]
    preferences: set[_Preference]
    logger: Logger
    verbose: bool
    def __init__(self, logger: Logger, verbose: bool = False) -> None: ...
    def close(self) -> None: ...
    def add_handler(self, handler: RequestHandler) -> None: ...
    def send(self, request: Request) -> Response: ...

def register_rh(handler: RequestHandler) -> RequestHandler: ...

class Features(enum.Enum):
    ALL_PROXY = 1
    NO_PROXY = 2

class RequestHandler(abc.ABC, metaclass=abc.ABCMeta):
    headers: HTTPHeaderDict | dict[str, str]
    cookiejar: YoutubeDLCookieJar | None
    timeout: float | int
    proxies: Mapping[str, Any] | dict[str, Any]
    source_address: str | None
    verbose: bool
    prefer_system_certs: bool
    verify: bool
    legacy_ssl_support: bool
    def __init__(
        self,
        *,
        logger: _YDLLogger,
        headers: HTTPHeaderDict | Mapping[str, str] | None = None,
        cookiejar: YoutubeDLCookieJar | None = None,
        timeout: float | None = None,
        proxies: Mapping[str, Any] | None = None,
        source_address: str | None = None,
        verbose: bool = False,
        prefer_system_certs: bool = False,
        client_cert: dict[str, str | None] | None = None,
        verify: bool = True,
        legacy_ssl_support: bool = False,
        **_: Unused,
    ) -> None: ...
    def validate(self, request: Request) -> None: ...
    def send(self, request: Request) -> Response: ...
    def close(self) -> None: ...
    @property
    def RH_NAME(cls) -> str: ...
    @property
    def RH_KEY(cls) -> str: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *args: object) -> None: ...

class Request:
    proxies: Mapping[str, Any] | dict[str, Any]
    extensions: Mapping[str, Any] | dict[str, Any]
    def __init__(
        self,
        url: str,
        data: _RequestData | None = None,
        headers: HTTPHeaderDict | Mapping[str, str] | None = None,
        proxies: Mapping[str, Any] | None = None,
        query: Mapping[str, str] | None = None,
        method: str | None = None,
        extensions: Mapping[str, Any] | None = None,
    ) -> None: ...
    @property
    def url(self) -> str: ...
    @url.setter
    def url(self, url: str) -> None: ...
    @property
    def method(self) -> str: ...
    @method.setter
    def method(self, method: str) -> None: ...
    @property
    def data(self) -> _RequestData | io.IOBase: ...
    @data.setter
    def data(self, data: _RequestData) -> None: ...
    @property
    def headers(self) -> HTTPHeaderDict | dict[str, str]: ...
    @headers.setter
    def headers(self, new_headers: Mapping[str, str] | HTTPHeaderDict) -> None: ...
    def update(
        self,
        url: str | None = None,
        data: str | None = None,
        headers: HTTPHeaderDict | Mapping[str, str] | None = None,
        query: Mapping[str, str] | None = None,
        extensions: Mapping[str, Any] | None = None,
    ) -> None: ...
    def copy(self) -> Self: ...

def HEADRequest(
    url: str,
    data: _RequestData | None = None,
    headers: HTTPHeaderDict | Mapping[str, str] | None = None,
    proxies: Mapping[str, Any] | None = None,
    query: Mapping[str, str] | None = None,
    *,
    method: str = "HEAD",
    extensions: Mapping[str, Any] | None = None,
) -> Request: ...
def PATCHRequest(
    url: str,
    data: _RequestData | None = None,
    headers: HTTPHeaderDict | Mapping[str, str] | None = None,
    proxies: Mapping[str, Any] | None = None,
    query: Mapping[str, str] | None = None,
    *,
    method: str = "PATCH",
    extensions: Mapping[str, Any] | None = None,
) -> Request: ...
def PUTRequest(
    url: str,
    data: _RequestData | None = None,
    headers: HTTPHeaderDict | Mapping[str, str] | None = None,
    proxies: Mapping[str, Any] | None = None,
    query: Mapping[str, str] | None = None,
    *,
    method: str = "PUT",
    extensions: Mapping[str, Any] | None = None,
) -> Request: ...

class Response(io.IOBase):
    fp: io.IOBase
    headers: Message
    status: int
    url: str
    reason: str | None
    extensions: Mapping[str, Any] | dict[str, Any]
    def __init__(
        self,
        fp: io.IOBase,
        url: str,
        headers: Mapping[str, str],
        status: int = 200,
        reason: str | None = None,
        extensions: Mapping[str, Any] | dict[str, Any] | None = None,
    ) -> None: ...
    def readable(self) -> bool: ...
    def read(self, amt: int | None = None) -> bytes: ...
    def close(self) -> None: ...
    def get_header(self, name: str, default: str | None = None) -> str | None: ...



from typing import Any, TypeVar

_S = TypeVar("_S")

class Settings(dict[str, Any]):
    app_id: str
    settings_directory: str
    settings_file: str
    def __init__(self, app_id: str) -> None: ...
    def add_setting(self, setting_name: str, setting_type: type[_S] = ..., default: _S | None = None) -> None: ...
    def load_settings(self) -> None: ...
    def save_settings(self) -> None: ...
    def __getattr__(self, setting_name: str) -> Any: ...
    def __setattr__(self, setting_name: str, setting_value: Any) -> None: ...
